#coding:utf-8

from django.db.models import signals as model_signals

try:
    import mptt
except ImportError:
    mptt = None


def disable_mptt_signals(model):
    '''
    Метод выключения обработки сигналов обработки дерева
    '''
    # Начиная с версии 0.5 сигналов в MPTT больше нет
    if mptt.VERSION < (0, 5):
        from mptt import signals as mptt_signals
        model_signals.pre_save.disconnect(
            receiver=mptt_signals.pre_save, sender=model)


def enable_mptt_signals(model):
    '''
    Метод включения обработки сигналов обработки дерева
    '''
    if mptt.VERSION < (0, 5):
        from mptt import signals as mptt_signals
        model_signals.pre_save.connect(
            receiver=mptt_signals.pre_save, sender=model)


def rebuild_mptt_tree(
        model, manage_mptt_signals=True, query_manager='objects'):
    """
    Метод пересчета атрибутов MPTT-модели. Может использоваться:
    - при включении новой модели в MPTT
    - при массовой загрузке данных

    Параметры:
        model - сама пересчитываемая модель
        manage_mptt_signals - признак выполнения отключения/включения
            сигналов MPTT для обработки дерева,
            иначе их надо отключать/включать вручную в вызывающем методе
        query_manager - вместо штатного менеджера запросов objects
            можно использовать собственный
    """

    import warnings
    warnings.warn(
        (
            'Use MyObj.tree.rebuild() instead '
            'http://django-mptt.github.com/django-mptt/mptt.managers'
            '.html?highlight=rebuild#mptt.managers.TreeManager.rebuild'
        ),
        DeprecationWarning,
        stacklevel=2
    )
    model_manager = getattr(model, query_manager)

    def build_node(model, opts, parent_id, tree_id, left, level):
        qs = model_manager.filter(**{opts.parent_attr: parent_id})
        #print 'tree=',tree_id,'left=',left,'level=',level
        right = left
        for node in qs:
            setattr(node, opts.tree_id_attr, tree_id)
            setattr(node, opts.left_attr, right+1)
            setattr(node, opts.level_attr, level+1)
            right = build_node(model, opts, node.pk, tree_id, right+1, level+1)
            setattr(node, opts.right_attr, right)
            node.save()
        right += 1
        return right

    if manage_mptt_signals:
        disable_mptt_signals(model)

    opts = model._meta
    qs = model_manager.filter(**{'%s__isnull' % opts.parent_attr: True})
    tree_id = 1
    #l = len(qs)
    for node in qs:
        #print 'tree=',tree_id, l
        setattr(node, opts.tree_id_attr, tree_id)
        setattr(node, opts.left_attr, 1)
        setattr(node, opts.level_attr, 0)
        right = build_node(model, opts, node.pk, tree_id, 1, 0)
        setattr(node, opts.right_attr, right)
        node.save()
        tree_id += 1

    if manage_mptt_signals:
        enable_mptt_signals(model)



#coding:utf-8
"""
Created on 29.01.2011

@author: akvarats
"""

from django.db import transaction
from django.dispatch import Signal

from m3 import RelatedError
from m3.actions import (
    ActionPack, Action, PreJsonResult, OperationResult, ACD, utils)
from m3.actions.packs import ListDeleteRowAction
from m3.actions.results import ActionResult
from m3.actions.interfaces import IMultiSelectablePack
from m3_ext.ui.results import ExtUIScriptResult
from m3_ext.ui.misc.store import ExtJsonStore
from m3_ext.ui.windows.complex import ExtDictionaryWindow
from m3_ext.ui.containers import ExtPagingBar
from m3.db import BaseObjectModel, safe_delete


class TreeGetNodesAction(Action):
    """
    Вызывает функцию получения узлов дерева у родительского пака
    """
    url = '/nodes'

    def context_declaration(self):
        return [
            ACD(name='node', type=int, required=True, default=0),
            ACD(name='filter', type=str, required=True, default=0),
        ]

    def run(self, request, context):
        parent_id = context.node if context.node > 0 else None
        return PreJsonResult(
            self.parent.get_nodes(request, context, parent_id, context.filter))


class TreeGetNodesLikeRows(Action):
    """
    Возвращает узлы дерева как список. Используется для автокомплита.
    """
    url = '/nodes_like_rows$'

    def context_declaration(self):
        return [
            ACD(name='filter', default='', type=str, required=True),
            ACD(name='branch_id', default=0, type=int, required=True)
        ]

    def run(self, request, context):
        result = self.parent.get_nodes_like_rows(
            request, context, context.filter, context.branch_id)
        return PreJsonResult(result)


class TreeGetNodeAction(Action):
    """
    Вызывает функцию получения узла дерева
    (нужно для редактирования в карточке)
    """
    url = '/node$'

    def run(self, request, context):
        id = utils.extract_int(request, 'id')
        result = self.parent.get_node(id)
        return PreJsonResult(result)


class TreeSaveNodeAction(Action):
    """
    Вызывает функцию сохранения узла дерева.
    """
    url = '/save_node$'

    def run(self, request, context):
        # Создаем форму для биндинга к ней
        win = self.parent.edit_node_window()
        win.form.bind_to_request(request)
        # Получаем наш объект по id
        id = utils.extract_int(request, 'id')
        obj = self.parent.get_node(id)
        # Биндим форму к объекту
        win.form.to_object(obj)
        result = self.parent.validate_node(obj, request)
        if result:
            assert isinstance(result, ActionResult)
            return result

        return self.parent.save_node(obj)


class TreeDeleteNodeAction(Action):
    """
    Получает узел из запроса и отправляет его на удаление
    """
    url = '/delete_node$'

    def run(self, request, context):
        id = utils.extract_int(request, self.parent.contextTreeIdName)
        obj = self.parent.get_node(id)
        return self.parent.delete_node(obj)


class ListGetRowsAction(Action):
    """
    Возвращает элементы принадлежание к узлу дерева
    в готовом для сериализации виде
    """
    url = '/rows$'

    def run(self, request, context):
        parent_id = utils.extract_int(request, 'id')
        offset = utils.extract_int(request, 'start')
        limit = utils.extract_int(request, 'limit')
        filter = request.REQUEST.get('filter')
        result = self.parent.get_rows(
            request, context, parent_id, offset, limit, filter)
        return PreJsonResult(result)


class ListGetRowAction(Action):
    url = '/item$'

    def run(self, request, context):
        id = utils.extract_int(request, 'id')
        result = self.parent.get_row(id)
        return PreJsonResult(result)


class ListSaveRowAction(Action):
    url = '/row$'

    def run(self, request, context):
        obj = utils.bind_request_form_to_object(
            request, self.parent.get_row, self.parent.edit_window)
        result = self.parent.validate_row(obj, request)
        if result:
            assert isinstance(result, ActionResult)
            return result
        return self.parent.save_row(obj)


class ListLastUsedAction(Action):
    url = '/last-rows$'

    def run(self, request, context):
        return PreJsonResult(self.parent.get_last_used(self))


class ListEditRowWindowAction(Action):
    """
    Экшен создает окно для редактирования элемента справочника (списка)
    """
    url = '/grid_edit_window$'

    def run(self, request, context):
        base = self.parent
        win = utils.bind_object_from_request_to_form(
            request, base.get_row, base.edit_window)
        win.orig_request = request
        win.orig_context = context
        if not win.title:
            win.title = base.title
        win.form.url = base.save_row_action.get_absolute_url()

        # проверим право редактирования
        if not self.parent.has_perm(request, self.parent.PERM_EDIT):
            win.make_read_only(
                access_off=True, exclude_list=['cancel_btn', 'close_btn'])

        return ExtUIScriptResult(base.get_edit_window(win))


class ListNewRowWindowAction(Action):
    """
    Экшен для создания нового элемента списка
    """
    url = '/grid_new_window$'

    def context_declaration(self):
        return [
            ACD(name=self.parent.contextTreeIdName,
                type=int,
                required=True,
                verbose_name=u'Код группы')
        ]

    def run(self, request, context):
        base = self.parent
        # Создаем новую группу и биндим ее к форме
        obj = base.get_row()
        setattr(
            obj,
            base.list_parent_field + '_id',
            getattr(context, base.contextTreeIdName)
        )
        win = base.edit_window(create_new=True)
        win.orig_request = request
        win.orig_context = context
        win.form.from_object(obj)
        # Донастраиваем форму
        if not win.title:
            win.title = base.title
        win.form.url = base.save_row_action.get_absolute_url()

        return ExtUIScriptResult(base.get_edit_window(win))


class TreeEditNodeWindowAction(Action):
    """
    Экшен создает окно для редактирования узла дерева
    """
    url = '/node_edit_window$'

    def run(self, request, context):
        base = self.parent
        win = utils.bind_object_from_request_to_form(
            request, base.get_node,
            base.edit_node_window, base.contextTreeIdName)
        win.orig_request = request
        win.orig_context = context
        if not win.title:
            win.title = base.title
        win.form.url = base.save_node_action.get_absolute_url()

        # проверим право редактирования
        if not self.parent.has_perm(request, self.parent.PERM_EDIT):
            win.make_read_only(
                access_off=True, exclude_list=['close_btn', 'cancel_btn'])

        # У окна может быть процедура доп. конфигурации под конкретный справочник
        if hasattr(win, 'configure_for_dictpack') and callable(
                win.configure_for_dictpack):
            win.configure_for_dictpack(
                action=self, pack=self.parent,
                request=request, context=context)

        return ExtUIScriptResult(base.get_node_edit_window(win))


class TreeNewNodeWindowAction(Action):
    """
    Экшен создает окно для создания нового узла дерева
    """
    url = '/node_new_window$'

    def run(self, request, context):
        base = self.parent
        # Получаем id родительской группы.
        # Если приходит не валидное значение, то создаем узел в корне
        parent_id = utils.extract_int(request, base.contextTreeIdName)
        if parent_id < 1:
            parent_id = None
        # Создаем новую группу и биндим ее к форме
        obj = base.get_node()
        obj.parent_id = parent_id
        win = base.edit_node_window(create_new=True)
        win.orig_request = request
        win.orig_context = context
        win.form.from_object(obj)
        # Донастраиваем форму
        if not win.title:
            win.title = base.title
        win.form.url = base.save_node_action.get_absolute_url()

        return ExtUIScriptResult(base.get_node_edit_window(win))


class TreeDragAndDropAction(Action):
    """
    Экшен перетаскивает узел дерева внутри самого дерева
    """
    url = '/drag_node$'

    def run(self, request, context):
        id = utils.extract_int(request, 'id')
        dest_id = utils.extract_int(request, 'dest_id')
        return self.parent.drag_node(id, dest_id)


class ListDragAndDropAction(Action):
    """
    Экшен перетаскивает запись из списка в другой узел дерева
    """
    url = '/drag_item$'

    def run(self, request, context):
        ids = utils.extract_int_list(request, 'id')
        dest_id = utils.extract_int(request, 'dest_id')
        return self.parent.drag_item(ids, dest_id)


class ListWindowAction(Action):
    """
    Экшен создает и настраивает окно справочника
    в режиме редактирования записей
    """
    url = '/get_list_window$'

    def create_window(self, request, context, mode):
        """
        Создаем и настраиваем окно
        """
        base = self.parent
        win = self.parent.list_window(title=base.title, mode=mode)
        win.height, win.width = base.height, base.width
        win.min_height, win.min_width = base.height, base.width

        if base.list_model:
            win.init_grid_components()
            if base.list_paging:
                win.grid.bottom_bar = ExtPagingBar(page_size=25)
        win.init_tree_components()
        win.tree.width = base.tree_width
        win.tree.root_text = base.title
        win.contextTreeIdName = base.contextTreeIdName

        return win

    def create_columns(self, control, columns):
        """
        Добавляем отображаемые колонки. См. описание в базовом классе!
        """
        for column in columns:
            if isinstance(column, tuple):
                column_params = {
                    'data_index': column[0],
                    'header': column[1]
                }
                if len(column) > 2:
                    column_params['width'] = column[2]
            elif isinstance(column, dict):
                column_params = column
            else:
                raise Exception('Incorrect parameter column.')
            control.add_column(**column_params)

    def configure_list(self, win, request, context):
        """
        Настраивает грид (список элементов)
        """
        base = self.parent
        if base.list_model:
            self.create_columns(win.grid, base.list_columns)

        # Устанавливаем источники данных
        if base.list_model:
            grid_store = ExtJsonStore(
                url=base.rows_action.get_absolute_url(), auto_load=True)
            grid_store.total_property = 'total'
            grid_store.root = 'rows'
            win.grid.set_store(grid_store)
        # Доступны 3 события для грида: создание нового элемента,
        # редактирование или удаление имеющегося
        if base.list_model and not base.list_readonly:
            win.url_new_grid = base.new_grid_window_action.get_absolute_url()
            win.url_edit_grid = base.edit_grid_window_action.get_absolute_url()
            win.url_delete_grid = base.delete_row_action.get_absolute_url()
            # Драг&Дроп
            if not base.tree_readonly and base.list_drag_and_drop:
                win.url_drag_grid = base.drag_list.get_absolute_url()

    def configure_tree(self, win, request, context):
        """
        Настраивает дерево групп
        """
        base = self.parent
        # Добавляем отображаемые колонки
        for column in base.tree_columns:
            if isinstance(column, tuple):
                column_params = {
                    'data_index': column[0],
                    'header': column[1]
                }
                if len(column) > 2:
                    if column[2] == 0:
                        column_params['hidden'] = True
                    else:
                        column_params['width'] = column[2]
                else:
                    column_params['width'] = 10
            elif isinstance(column, dict):
                column_params = column
            else:
                raise Exception('Incorrect parameter column.')
            win.tree.add_column(**column_params)
        # Устанавливаем источники данных
        win.tree.url = base.nodes_action.get_absolute_url()
        # События для дерева
        if not base.tree_readonly:
            # Доступны 3 события для дерева: создание нового узла,
            # редактирование или удаление имеющегося
            win.url_new_tree = base.new_node_window_action.get_absolute_url()
            win.url_edit_tree = base.edit_node_window_action.get_absolute_url()
            win.url_delete_tree = base.delete_node_action.get_absolute_url()
            # Драг&Дроп
            if base.list_drag_and_drop:
                win.url_drag_tree = base.drag_tree.get_absolute_url()

    def configure_other(self, win, request, context):
        pass

    def run(self, request, context):
        win = self.create_window(request, context, 0)
        win.orig_request = request
        win.orig_context = context
        self.configure_tree(win, request, context)
        self.configure_list(win, request, context)
        self.configure_other(win, request, context)
        win = self.parent.get_list_window(win)

        # проверим право редактирования
        if not self.parent.has_perm(request, self.parent.PERM_EDIT):
            win.make_read_only()

        return ExtUIScriptResult(win)


class SelectWindowAction(ListWindowAction):
    """
    Экшен создает и настраивает окно справочника в режиме выбора
    """
    url = '/get_select_window$'
    mode = ExtDictionaryWindow.SELECT_MODE

    def run(self, request, context):
        # Создаем окно выбора
        win = self.create_window(request, context, self.mode)
        win.orig_request = request
        win.orig_context = context
        win.modal = True
        # Добавляем отображаемые колонки
        base = self.parent
        self.configure_list(win, request, context)
        self.configure_tree(win, request, context)

        win.column_name_on_select = base.column_name_on_select

        win = self.parent.get_select_window(win)
        win.contextTreeIdName = base.contextTreeIdName

        # проверим право редактирования
        if not self.parent.has_perm(request, self.parent.PERM_EDIT):
            win.make_read_only()

        return ExtUIScriptResult(win)


class MultiSelectWindowAction(SelectWindowAction):
    url = '/get_multis_select_window$'
    mode = ExtDictionaryWindow.MULTI_SELECT_MODE


class BaseTreeDictionaryActions(ActionPack, IMultiSelectablePack):
    """
    Пакет с действиями, специфичными для работы с иерархическими справочниками
    """
    # Список колонок состоящий из:
    # 1. вариант (классический)
    #    list_actions = [('code', u'Код'), ('name', u'Наименование')]
    # 2. вариант (классический расширенный)
    #   - третьим элементом в кортеже идет ширина
    #    list_actions = [('code', u'Код', 100), ('name', u'Наименование')]
    # 3. вариант (универсальный)
    #    list_actions = [{'name': 'code','header':u'Код', 'width': 100}, (...)]
    list_columns = []

    # Заголовок окна справочника
    title = ''
    # Окно редактирования узла дерева
    edit_node_window = None
    # Окно редактирования элемента списка
    edit_window = None
    # Окно самого справочника
    list_window = ExtDictionaryWindow
    # Ширина и высота окна
    width = 600
    height = 400
    tree_width = 200

    contextTreeIdName = 'id'

    # права доступа для базовых справочников
    PERM_EDIT = 'edit'
    sub_permissions = {PERM_EDIT: u'Редактирование справочника'}

    # Колонка для выбора по умолчанию
    column_name_on_select = 'name'

    def __init__(self):
        super(BaseTreeDictionaryActions, self).__init__()

        # Экшены отдающие данные
        self.nodes_action = TreeGetNodesAction()
        self.node_action = TreeGetNodeAction()
        self.rows_action = ListGetRowsAction()
        self.row_action = ListGetRowAction()
        self.nodes_like_rows_action = TreeGetNodesLikeRows()
        self.last_used_action = ListLastUsedAction()
        self.actions.extend([
            self.nodes_action,
            self.node_action,
            self.rows_action,
            self.row_action,
            self.last_used_action,
            self.nodes_like_rows_action
        ])

        # Окна самого справочника
        self.list_window_action = ListWindowAction()
        self.select_window_action = SelectWindowAction()
        self.multi_select_window_action = MultiSelectWindowAction()
        self.actions.extend([
            self.list_window_action,
            self.select_window_action,
            self.multi_select_window_action
        ])

        # Адреса экшенов списка
        self.new_grid_window_action = ListNewRowWindowAction()
        self.edit_grid_window_action = ListEditRowWindowAction()
        self.save_row_action = ListSaveRowAction()
        self.delete_row_action = ListDeleteRowAction()
        self.actions.extend([
            self.new_grid_window_action,
            self.edit_grid_window_action,
            self.save_row_action,
            self.delete_row_action
        ])

        # Драг&Дроп
        self.drag_tree = TreeDragAndDropAction()
        self.drag_list = ListDragAndDropAction()
        self.actions.extend([self.drag_tree, self.drag_list])

        # Адреса экшенов дерева
        self.new_node_window_action = TreeNewNodeWindowAction()
        self.edit_node_window_action = TreeEditNodeWindowAction()
        self.save_node_action = TreeSaveNodeAction()
        self.delete_node_action = TreeDeleteNodeAction()
        self.actions.extend([
            self.new_node_window_action,
            self.edit_node_window_action,
            self.save_node_action,
            self.delete_node_action
        ])

    #========================== ДЕРЕВО ===========================

    def get_nodes(self, parent_id, filter):
        raise NotImplementedError()

    def get_node(self, id):
        raise NotImplementedError()

    def validate_node(self, obj, request):
        pass

    def save_node(self, obj, parent_id):
        raise NotImplementedError()

    def delete_node(self, obj):
        raise NotImplementedError()

    def get_default_action(self):
        """
        Возвращает экшн по-умолчанию (для встраивания в UI)
        """
        return self.list_window_action

    #================ ФУНКЦИИ ВОЗВРАЩАЮЩИЕ АДРЕСА ===============
    #ISelectablePack
    def get_select_url(self):
        """
        Возвращает адрес формы списка элементов справочника.
        """
        return self.select_window_action.get_absolute_url()

    #IMultiSelectablePack
    def get_multi_select_url(self):
        return self.multi_select_window_action.get_absolute_url()

    def get_list_url(self):
        """
        Возвращает адрес формы списка элементов справочника.
        """
        return self.list_window_action.get_absolute_url()

    #ISelectablePack
    def get_edit_url(self):
        """
        Возвращает адрес формы редактирования элемента справочника.
        """
        return self.edit_grid_window_action.get_absolute_url()

    def get_edit_node_url(self):
        """
        Возвращает адрес формы редактирования группы справочника.
        """
        return self.edit_node_window_action.get_absolute_url()

    def get_rows_url(self):
        """
        Возвращает адрес по которому запрашиваются элементы грида.
        """
        return self.rows_action.get_absolute_url()

    def get_nodes_url(self):
        """
        Возвращает адрес по которому запрашиваются группы дерева.
        """
        return self.nodes_action.get_absolute_url()

    def get_nodes_like_rows_url(self):
        """
        Возвращает адрес по которому запрашиваются группы дерева как список
        """
        return self.nodes_like_rows_action.get_absolute_url()

    #ISelectablePack
    def get_autocomplete_url(self):
        """
        Получить адрес для запроса элементов
        подходящих введенному в поле тексту
        """
        return self.get_nodes_like_rows_url()

    #=================== ИЗМЕНЕНИЕ ДАННЫХ =======================

    def get_rows(self, offset, limit, filter, parent_id):
        raise NotImplementedError()

    def get_row(self, id):
        raise NotImplementedError()

    def validate_row(self, obj, request):
        pass

    def save_row(self, obj, parent_id):
        raise NotImplementedError()

    def delete_row(self, obj):
        raise NotImplementedError()

    #ISelectablePack
    def get_display_text(self, key, attr_name=None):
        """
        Получить отображаемое значение записи
        (или атрибута attr_name) по ключу key
        """
        raise NotImplementedError()

    #ISelectablePack
    def get_record(self, key):
        """
        Получить записи по ключу key
        """
        raise NotImplementedError()

    #IMultiSelectablePack
    def get_display_dict(self, key, value_field='id', display_field='name'):
        """
        Получить список словарей, необходимый для представления выбранных
        значений ExtMultiSelectField
        """
        raise NotImplementedError()

    #============ ДЛЯ ИЗМЕНЕНИЯ ОКОН ВЫБОРА НА ХОДУ ==============
    def get_select_window(self, win):
        return win

    def get_list_window(self, win):
        return win

    #======================= Drag&Drop ===========================
    def drag_node(self, id, dest_id):
        raise NotImplementedError()

    def drag_item(self, id, dest_id):
        raise NotImplementedError()

    #============ ДЛЯ ИЗМЕНЕНИЯ ОКОН РЕДАКТИРОВАНИЯ НА ХОДУ ======
    def get_edit_window(self, win):
        return win

    def get_node_edit_window(self, win):
        return win


class BaseTreeDictionaryModelActions(BaseTreeDictionaryActions):
    """
    Класс реализует действия над иерархическим справочником,
    основанном на моделе
    """
    # Признак возвращения всех узлов дерева
    ALL_ROWS = -1

    # Поля для поиска по умолчанию.
    DEFAULT_FILTER_FIELDS = ['code', 'name']

    # Настройки для модели дерева
    tree_model = None  # Сама модель дерева
    tree_filter_fields = []  # Поля по которым производится поиск в дереве
    tree_columns = []  # Список из кортежей с параметрами выводимых в дерево колонок
    tree_parent_field = 'parent'  # Имя поля ссылающегося на группу
    tree_readonly = False  # Если истина, то адреса экшенов дереву не назначаются
    tree_order_field = ''

    # Настройки модели списка
    list_model = None  # Не обязательная модель списка связанного с деревом
    list_columns = []  # Список из кортежей с параметрами выводимых в грид колонок
    filter_fields = []  # Поля по которым производится поиск в списке
    list_parent_field = 'parent'  # Имя поля ссылающегося на группу
    list_readonly = False  # Если истина, то адреса экшенов гриду не назначаются
    list_drag_and_drop = True  # Разрешает перетаскивание элементов из грида в другие группы дерева
    list_order_field = ''
    list_paging = True

    # Порядок сортировки элементов списка. Работает следующим образом:
    # 1. Если в list_columns модели списка есть поле code,
    #   то устанавливается сортировка по возрастанию этого поля;
    # 2. Если в list_columns модели списка нет поля code, но есть поле name,
    #   то устанавливается сортировка по возрастанию поля name;
    # Пример list_sort_order = ['code', '-name']
    list_sort_order = []
    tree_sort_order = None

    def __init__(self):
        super(BaseTreeDictionaryModelActions, self).__init__()
        # Установка значений по умолчанию для поиска и сортировки
        if self.list_model:
            self.filter_fields = self._default_list_search_filter()
            self.list_sort_order = self._default_order()
        if self.tree_model:
            self.tree_filter_fields = self._default_tree_search_filter()

    def get_nodes(self, request, context, parent_id, filter, branch_id=None):
        """
        Метод получения списка узлов дерева, которые
        """
        # parent_id - это элемент, который раскрывается,
        # поэтому для него фильтр ставить не надо, иначе фильтруем
        # branch_id - это элемент ограничивающий дерево,
        # т.е. должны возвращаться только дочерние ему элементы
        if filter and not parent_id:
            filter_dict = utils.create_search_filter(
                filter, self.tree_filter_fields)
            nodes = utils.fetch_search_tree(
                self.tree_model, filter_dict, branch_id)
        else:
            if branch_id and hasattr(self.tree_model, 'get_descendants'):
                branch_node = self.tree_model.objects.get(id=branch_id)
                if parent_id:
                    query = branch_node.get_descendants().filter(
                        parent=parent_id)
                else:
                    query = branch_node.get_children()
            else:
                query = self.tree_model.objects.filter(parent=parent_id)
            query = utils.apply_sort_order(
                query, self.tree_columns, self.tree_sort_order)
            query = utils.detect_related_fields(query, self.list_columns)

            # кастомная функция модификации запроса
            # при реализации контестных справочников в большинстве случаев
            # достаточно будет просто переопределить данную функцию
            query = self.modify_get_nodes(query, request, context)

            nodes = list(query)
            # Если имеем дело с листом, нужно передавать параметр leaf = true
            for node in nodes:
                if not self.tree_model.objects.filter(parent=node.id).exists():
                    node.leaf = 'true'

        # генерируем сигнал о том, что узлы дерева подготовлены
        nodes_prepared.send(sender=self.__class__, nodes=nodes)
        return nodes

    def modify_get_nodes(self, query, request, context):
        return query

    def get_rows(self, request, context, parent_id, offset, limit, filter):
        # если справочник состоит только из дерева и у него просят запись,
        # то надо брать из модели дерева
        # TODO: возможно это не надо было делать - раз не туда обратились,
        # значит сами виноваты
        if self.list_model:
            query = None
            if parent_id == BaseTreeDictionaryModelActions.ALL_ROWS:
                # отображаются все данные
                query = self.list_model.objects
            else:
                # отображаются данные с фильтрацией по значению parent_id
                query = self.list_model.objects.filter(
                    **{self.list_parent_field: parent_id})

            # Подтягиваем группу, т.к. при сериализации она требуется
            query = query.select_related(self.list_parent_field)
            query = utils.apply_sort_order(
                query, self.list_columns, self.list_sort_order)
            query = utils.apply_search_filter(
                query, filter, self.filter_fields)
            query = utils.detect_related_fields(query, self.list_columns)
            # Для работы пейджинга нужно передавать общее количество записей
            query = self.modify_get_rows(query, request, context)
            total = query.count()
            # Срез данных для страницы
            if limit > 0:
                query = query[offset: offset + limit]

            result = {'rows': list(query.all()), 'total': total}
            return result
        else:
            return self.get_nodes(request, context, parent_id, filter)

    def modify_get_rows(self, query, request, context):
        """
        метод для переопределения запроса на получение данных справочника
        """
        return query

    def _get_model_fieldnames(self, model):
        """
        Возвращает имена всех полей модели
        """
        return [field.attname for field in model._meta.local_fields]

    def _default_order(self):
        """
        Устанавливаем параметры сортировки
        по умолчанию 'code' и 'name' в случае,
        если у модели есть такие поля
        """
        filter_order = self.list_sort_order
        if not filter_order:
            filter_order = []
            all_fields = self._get_model_fieldnames(self.list_model)
            filter_order.extend([
                field for field in ('code', 'name', 'id')
                if field in all_fields
            ])
        return filter_order

    def _default_tree_search_filter(self):
        """
        Если поля для поиска не заданы, то возвращает список из полей модели
        по которым будет производиться поиск.
        По умолчанию берутся код и наименование
        """
        if not self.tree_filter_fields:
            assert self.tree_model, 'Tree model is not defined!'
            for field_name in self._get_model_fieldnames(self.tree_model):
                if field_name in self.DEFAULT_FILTER_FIELDS:
                    self.tree_filter_fields.append(field_name)

        return self.tree_filter_fields

    def _default_list_search_filter(self):
        """
        Если поля для поиска не заданы, то возвращает список из полей модели
        по которым будет производиться поиск.
        По умолчанию берутся код и наименование
        """
        if not self.filter_fields:
            assert self.list_model, 'List model is not defined!'
            for field_name in self._get_model_fieldnames(self.list_model):
                if field_name in self.DEFAULT_FILTER_FIELDS:
                    self.filter_fields.append(field_name)

        return self.filter_fields

    def get_nodes_like_rows(self, request, context, filter, branch_id=None):
        """
        Возвращаются узлы дерева, предствленные в виде общего списка
        """
        # branch_id - это элемент ограничивающий дерево,
        # т.е. должны возвращаться только дочерние ему элементы
        if filter:
            filter_dict = utils.create_search_filter(
                filter, self.tree_filter_fields)
            if branch_id and hasattr(self.tree_model, 'get_descendants'):
                branch_node = self.tree_model.objects.get(id=branch_id)
                nodes = branch_node.get_descendants().filter(
                    filter_dict).select_related('parent')
            else:
                nodes = self.tree_model.objects.filter(
                    filter_dict).select_related('parent')
        else:
            if branch_id and hasattr(self.tree_model, 'get_descendants'):
                branch_node = self.tree_model.objects.get(id=branch_id)
                nodes = branch_node.get_descendants()
            else:
                nodes = self.tree_model.objects.all()
        # кастомная функция модификации запроса
        # при реализации контестных справочников в большинстве случаев
        # достаточно будет просто переопределить данную функцию
        nodes = self.modify_get_nodes(nodes, request, context)
        # Для работы пейджинга нужно передавать общее количество записей
        total = len(nodes)
        result = {'rows': list(nodes), 'total': total}
        return result

    def _get_obj(self, model, id):
        """
        Возвращает запись заданной модели model по id
        Если id нет, значит нужно создать новый объект
        """
        assert isinstance(id, int)
        if id == 0:
            obj = model()
        else:
            try:
                obj = model.objects.get(id=id)
            except model.DoesNotExist:
                return None
        return obj

    def get_node(self, id=0):
        return self._get_obj(self.tree_model, id)

    def get_row(self, id=0):
        # если справочник состоит только из дерева и у него просят запись,
        # то надо брать из модели дерева
        # TODO: это надо было для элемента выбора из справочника
        # - он не знает откуда ему взять запись и всегда вызывает get_row.
        # может надо было как-то по-другому это решить
        if self.list_model:
            return self._get_obj(self.list_model, id)
        else:
            return self.get_node(id)

    def save_row(self, obj):
        obj.save()
        return OperationResult(success=True)

    def save_node(self, obj):
        obj.save()
        return OperationResult(success=True)

    def delete_row(self, objs):
        # Такая реализация обусловлена тем,
        # что IntegrityError невозможно отловить
        # до завершения транзакции, и приходится оборачивать транзакцию.
        @transaction.commit_on_success
        def delete_row_in_transaction(self, objs):
            message = ''
            if len(objs) == 0:
                message = u'Элемент не существует в базе данных.'
            else:
                for obj in objs:
                    if (
                        isinstance(obj, BaseObjectModel) or
                        (hasattr(obj, 'safe_delete') and
                            callable(obj.safe_delete))
                    ):
                        try:
                            obj.safe_delete()
                        except RelatedError, e:
                            message = e.args[0]
                    else:
                        if not safe_delete(obj):
                            message = (
                                u'Не удалось удалить элемент %s. '
                                u'Возможно на него есть ссылки.'
                            ) % obj.id
                            break
            return OperationResult.by_message(message)
        # Тут пытаемся поймать ошибку из транзакции.
        try:
            return delete_row_in_transaction(self, objs)
        except Exception, e:
            # Встроенный в Django IntegrityError не генерируется.
            # Кидаются исключения специфичные для каждого драйвера БД.
            # Но по спецификации PEP 249 все они называются IntegrityError
            if e.__class__.__name__ == 'IntegrityError':
                message = (
                    u'Не удалось удалить элемент. '
                    u'Возможно на него есть ссылки.'
                )
                return OperationResult.by_message(message)
            else:
                # все левые ошибки выпускаем наверх
                raise

    def delete_node(self, obj):
        """
        Удаление группы справочника.
        Нельзя удалять группу если у нее есть подгруппы,
        или если в ней есть элементы. Но даже после этого удалять
        группу можно только прямым запросом, т.к. мы не знаем заранее,
        кто на нее может ссылаться и кого зацепит каскадное удаление джанги.
        """
        message = ''
        if obj is None:
            message = u'Группа не существует в базе данных.'
        elif self.tree_model.objects.filter(
                **{self.tree_parent_field: obj}).exists():
            message = u'Нельзя удалить группу содержащую в себе другие группы.'
        elif self.list_model and self.list_model.objects.filter(
                **{self.list_parent_field: obj}).exists():
            message = u'Нельзя удалить группу содержащую в себе элементы.'
        elif not safe_delete(obj):
            message = (
                u'Не удалось удалить группу. Возможно на неё есть ссылки.')

        return OperationResult.by_message(message)

    def drag_node(self, id, dest_id):
        node = self.get_node(id)
        # Если id узла на который кидаем <1, значит это корень справочника
        if dest_id < 1:
            node.parent_id = None
        else:
            node.parent_id = dest_id
        node.save()
        return OperationResult()

    def drag_item(self, ids, dest_id):
        # В корень нельзя кидать простые элементы
        if dest_id < 1:
            return OperationResult.by_message(
                u'Нельзя перемещать элементы в корень справочника!')

        # Из грида в дерево одновременно
        # могут быть перенесены несколько элементов
        # Их id разделены запятыми
        for id in ids:
            row = self.get_row(id)
            row.parent_id = dest_id
            row.save()
        return OperationResult()

    #ISelectablePack
    def get_edit_url(self):
        """
        Получить адрес для запроса диалога редактирования выбранного элемента
        """
        # тут возможны варианты, когда pack используется без грида
        # в этом случае нужно возвращать ссылку
        # на редактирование элемента дерева
        # именно по этой причине была сделана
        # эта обертка над методом get_edit_url
        if self.list_model:
            return super(BaseTreeDictionaryModelActions, self).get_edit_url()
        elif self.tree_model:
            return self.get_edit_node_url()
        return None

    #ISelectablePack
    def get_display_text(self, key, attr_name=None):
        """
        Получить отображаемое значение записи
        (или атрибута attr_name) по ключу key
        """
        # тут возможны варианты, когда pack используется без грида
        # в этом случае нужно работать с элементом дерева
        row = self.get_record(key)
        if row is not None:
            name = attr_name if attr_name else self.column_name_on_select
            text = getattr(row, name)
            # getattr может возвращать метод, например verbose_name
            if callable(text):
                return text()
            else:
                return text
        return None

    #ISelectablePack
    def get_record(self, key):
        """ Получить запись по ключу key """
        row = None
        if self.list_model:
            row = self.get_row(key)
        elif self.tree_model:
            row = self.get_node(key)
        return row

    #IMultiSelectablePack
    def get_display_dict(self, key, value_field='id', display_field='name'):
        """
        Получить список словарей, необходимый для представления выбранных
        значений ExtMultiSelectField
        """
        items = []
        row = None
        if self.list_model:
            row = self.get_row(key)
        elif self.tree_model:
            row = self.get_node(key)
        if row is not None:
            keys = key if isinstance(key, (list, tuple,)) else [key, ]
            for key in keys:
                value = getattr(row, display_field, None)
                if value:
                    items.append({
                        value_field: key,
                        display_field: value,
                    })
        return items

#===============================================================================
# Сигналы, которые посылаются в процессе
# работы подсистемы древовидных справочника
#==============================================================================

# сигнал о том, что узлы дерева иерархического справочника подготовлены
nodes_prepared = Signal(providing_args=['nodes'])



#coding: utf-8
"""
Паки и экшены для работы со справочниками
+++++++++++++++++++++++++++++++++++++++++
"""
from django.db import transaction
from django.conf import settings
try:
    from django.utils.log import logger
except ImportError:
    from django.utils.log import getLogger
    logger = getLogger('django')

from m3.actions import (
    ActionPack, Action, PreJsonResult, OperationResult, ACD
)
from m3_ext.ui.windows.complex import ExtDictionaryWindow
from m3_ext.ui.misc.store import ExtJsonStore
from m3_ext.ui.containers import ExtPagingBar
from m3_ext.ui.results import ExtUIScriptResult
from m3.actions import utils
from m3.actions.results import ActionResult
from m3.db import BaseObjectModel, safe_delete
from m3 import RelatedError

try:
    from m3_audit import AuditManager
except ImportError:
    # При сборке документации, внешняя Django ничего не знает про m3_audit
    logger.warning('m3_audit import error')

from m3.actions.interfaces import ISelectablePack

MSG_DOESNOTEXISTS = (
    u'Запись справочника с id=%s не найдена в базе данных.<br/>'
    u'Возможно, она была удалена. Пожалуйста, обновите таблицу.'
)


class ObjectNotFound(Exception):
    """
    Исключение для виртуальных справочников. Аналог DoesNotExists у моделей.
    """
    pass


class DictListWindowAction(Action):
    """
    Действие, которое возвращает окно со списком элементов справочника.
    """
    url = '/list-window$'

    def create_window(self, request, context, mode):
        """
        Создаем и настраиваем окно
        """
        base = self.parent
        allow_copy = hasattr(base, 'allow_copy') and base.allow_copy
        win = base.list_form(mode=mode, title=base.title)
        win.allow_copy = allow_copy
        win.height, win.width = base.height, base.width
        win.min_height, win.min_width = base.height, base.width

        win.init_grid_components()
        if base.list_paging:
            win.grid.bottom_bar = ExtPagingBar(page_size=25)
        return win

    def create_columns(self, control, columns):
        """
        Добавляем отображаемые колонки. См. описание в базовом классе!
        """
        for column in columns:
            if isinstance(column, tuple):
                column_params = {
                    'data_index': column[0],
                    'header': column[1],
                    'sortable': True}

                if len(column) > 2:
                    column_params['width'] = column[2]
            elif isinstance(column, dict):
                column_params = column
            else:
                raise Exception('Incorrect parameter column.')
            control.add_column(**column_params)

    def configure_list(self, win):
        base = self.parent

        # Устанавливаем источники данных
        grid_store = ExtJsonStore(
            url=base.rows_action.get_absolute_url(),
            auto_load=True,
            remote_sort=True)
        grid_store.total_property = 'total'
        grid_store.root = 'rows'
        win.grid.set_store(grid_store)

        if not base.list_readonly:
            # Доступны 3 события: создание нового элемента,
            # редактирование или удаление имеющегося
            win.url_new_grid = base.edit_window_action.get_absolute_url()
            win.url_edit_grid = base.edit_window_action.get_absolute_url()
            win.url_delete_grid = base.delete_action.get_absolute_url()

            # Если разрешено копирование, то доступно ещё одно событие.
            if getattr(base, 'allow_copy', False):
                win.url_copy_grid = base.copy_action.get_absolute_url()

    def run(self, request, context):
        win = self.create_window(request, context, mode=0)
        self.create_columns(win.grid, self.parent.list_columns)
        self.configure_list(win)

        # проверим право редактирования
        if not self.parent.has_sub_permission(
                request.user, self.parent.PERM_EDIT, request):
            win.make_read_only()

        return ExtUIScriptResult(self.parent.get_list_window(win))


class DictSelectWindowAction(DictListWindowAction):
    """
    Действие, возвращающее окно с формой выбора из справочника
    """
    url = '/select-window$'

    def run(self, request, context):
        base = self.parent
        win = self.create_window(request, context, mode=1)
        win.modal = True
        self.create_columns(win.grid, self.parent.list_columns)
        self.configure_list(win)

        # M prefer 12.12.10 >
        # win.column_name_on_select = "name"
        #-----:
        win.column_name_on_select = base.column_name_on_select
        # prefer <

        # проверим право редактирования
        if not self.parent.has_sub_permission(
                request.user, self.parent.PERM_EDIT, request):
            win.make_read_only()

        return ExtUIScriptResult(self.parent.get_select_window(win))


class DictEditWindowAction(Action):
    """
    Редактирование элемента справочника
    """
    url = '/edit-window$'

    def context_declaration(self):
        return [ACD(name='id',
                    default=0,
                    type=int,
                    required=True,
                    verbose_name=u'id элемента справочника'),
                ACD(name='isGetData',
                    default=False,
                    type=bool,
                    required=True,
                    verbose_name=u'признак загрузки данных')]

    def run(self, request, context):
        base = self.parent
        is_get_data = context.isGetData
        # Получаем объект по id
        try:
            obj = base.get_row(context.id)
        except base._nofound_exception:
            return OperationResult.by_message(MSG_DOESNOTEXISTS % context.id)

        # Разница между новым и созданным объектов в том,
        # что у нового нет id или он пустой
        create_new = True
        if isinstance(obj, dict) and obj.get('id') is not None:
            create_new = False
        elif hasattr(obj, 'id') and getattr(obj, 'id') is not None:
            create_new = False
        if create_new and base.add_window:
            win = utils.bind_object_from_request_to_form(
                request, base.get_row, base.add_window)
        else:
            win = utils.bind_object_from_request_to_form(
                request, base.get_row, base.edit_window)

        if not win.title:
            win.title = base.title
        win.form.url = base.save_action.get_absolute_url()
        # укажем адрес для чтения данных
        win.data_url = base.edit_window_action.get_absolute_url()

        # проверим право редактирования
        if not self.parent.has_sub_permission(
                request.user, self.parent.PERM_EDIT, request):
            exclude_list = ['close_btn', 'cancel_btn']
            win.make_read_only(True, exclude_list)

        # У окна может быть процедура доп. конфигурации
        # под конкретный справочник
        if (hasattr(win, 'configure_for_dictpack') and
                callable(win.configure_for_dictpack)):
            win.configure_for_dictpack(action=self, pack=self.parent,
                                       request=request, context=context)

        if not is_get_data:
            # если запрашивали не данные - вернем окно
            return ExtUIScriptResult(base.get_edit_window(win))
        else:
            # если просили данные, то выжмем их из окна обратно в объект,
            # т.к. в окне могли быть и другие данных (не из этого объекта)
            data_object = {}
            # т.к. мы не знаем какие поля должны быть у объекта - создадим
            # все, которые есть на форме
            all_fields = win.form._get_all_fields(win)
            for field in all_fields:
                data_object[field.name] = None
            win.form.to_object(data_object)
            return PreJsonResult({'success': True, 'data': data_object})


class DictRowsAction(Action):
    """
    Действие, которое возвращает список записей справочника.
    Именно список записей, которые потом могут быть отображены в гриде.
    В качестве контекста выполнения может быть задано:
    a) текстовая строка с фильтром (для выполнения поиска);
    b) начальная позиция и смещение записей для пейджинга.
    """
    url = '/rows$'

    def run(self, request, context):
        offset = utils.extract_int(request, 'start')
        limit = utils.extract_int(request, 'limit')
        filter = request.REQUEST.get('filter')
        direction = request.REQUEST.get('dir')
        user_sort = request.REQUEST.get('sort')
        if direction == 'DESC':
            user_sort = '-' + user_sort
        dict_list = []
        for item in self.parent.list_columns:
            if isinstance(item, (list, tuple)):
                dict_list.append(item[0])
            elif isinstance(item, dict) and item.get('data_index'):
                dict_list.append(item['data_index'])

        if (hasattr(self.parent, 'modify_rows_query') and
                callable(self.parent.modify_rows_query)):
            rows = self.parent.get_rows_modified(
                offset, limit, filter, user_sort, request, context)
        else:
            rows = self.parent.get_rows(offset, limit, filter, user_sort)
        return PreJsonResult(
            rows, self.parent.secret_json, dict_list=dict_list)


class DictLastUsedAction(Action):
    """
    Действие, которое возвращает список последних использованных действий
    """
    url = '/last-rows$'

    def run(self, request, context):
        return PreJsonResult(self.parent.get_last_used(self))


class ListGetRowAction(Action):
    """
    Действие, которое отвечает за возврат данных для
    одного отдельно-взятой записи справочника
    """
    url = '/item$'

    def context_declaration(self):
        return [ACD(
            name='id',
            default=0,
            type=int,
            required=True,
            verbose_name=u'id элемента справочника')]

    def run(self, request, context):
        try:
            result = self.parent.get_row(context.id)
        except self.parent._nofound_exception:
            return OperationResult.by_message(MSG_DOESNOTEXISTS % context.id)
        return PreJsonResult(result)


class DictSaveAction(Action):
    """
    Действие выполняет сохранение записи справочника.
    """
    url = '/save$'

    def context_declaration(self):
        return [
            ACD(
                name='id',
                default=0,
                type=int,
                required=True,
                verbose_name=u'id элемента справочника')]

    def run(self, request, context):
        try:
            if not context.id and self.parent.add_window:
                obj = utils.bind_request_form_to_object(
                    request, self.parent.get_row, self.parent.add_window)
            else:
                obj = utils.bind_request_form_to_object(
                    request, self.parent.get_row, self.parent.edit_window)
        except self.parent._nofound_exception:
            return OperationResult.by_message(MSG_DOESNOTEXISTS % context.id)

        # Проверка корректности полей сохраняемого объекта
        result = self.parent.validate_row(obj, request)
        if result:
            assert isinstance(result, ActionResult)
            return result

        result = self.parent.save_row(obj)
        if isinstance(result, OperationResult) and result.success is True:
            # узкое место. после того, как мы переделаем работу экшенов,
            # имя параметра с идентификатором запси может уже называться не
            # id
            if 'm3_audit' in settings.INSTALLED_APPS:
                AuditManager().write(
                    'dict-changes',
                    user=request.user,
                    model_object=obj,
                    type='new' if not context.id else 'edit')
            context.id = obj.id
        return result


class ListDeleteRowAction(Action):
    url = '/delete_row$'

    def run(self, request, context):
        """
        Удаляться одновременно могут несколько объектов. Их ключи
        приходят разделенные запятыми.
        """
        ids = utils.extract_int_list(request, 'id')
        try:
            objs = [self.parent.get_row(id) for id in ids]
        except self.parent._nofound_exception:
            return OperationResult.by_message(MSG_DOESNOTEXISTS % id)

        result = self.parent.delete_row(objs)
        if (isinstance(result, OperationResult) and
                result.success is True and
                'm3_audit' in settings.INSTALLED_APPS):
            for obj in objs:
                AuditManager().write(
                    'dict-changes',
                    user=request.user,
                    model_object=obj,
                    type='delete')
        return result


class DictCopyAction(Action):
    """
    Копирование записи из справочника
    """

    url = '/copy$'

    def context_declaration(self):
        return [
            ACD(
                name='id',
                type=int,
                required=True,
                verbose_name=u'id элемент справочника'
            )]

    def run(self, request, context):
        """
        """
        base = self.parent

        win = utils.bind_object_from_request_to_form(
            request, base.get_row, base.edit_window, exclusion=['id'])

        if not win.title:
            win.title = base.title

        win.form.url = base.save_action.get_absolute_url()
        # укажем адрес для чтения данных
        win.data_url = base.edit_window_action.get_absolute_url()
        win.orig_request = request
        win.orig_context = context

        # У окна может быть процедура доп. конфигурации
        # под конкретный справочник
        if (hasattr(win, 'configure_for_dictpack') and
                callable(win.configure_for_dictpack)):
            win.configure_for_dictpack(
                action=self, pack=self.parent,
                request=request, context=context)

        return ExtUIScriptResult(base.get_edit_window(win))


class BaseDictionaryActions(ActionPack, ISelectablePack):
    """
    Пакет с действиями, специфичными для работы со справочниками
    """
    # Заголовок окна справочника
    title = ''  # для записи
    # Список колонок состоящий из кортежей (имя json поля, имя колонки в окне)
    list_columns = []

    # Окно для редактирования элемента справочника:
    add_window = None  # Нового
    edit_window = None  # Уже существующего

    # Класс отвечающие за отображение форм:
    list_form = ExtDictionaryWindow  # Форма списка
    select_form = ExtDictionaryWindow  # Форма выбора

    # Настройки секретности. Если стоит истина,
    # то в результат добавляется флаг секретности
    secret_json = False
    secret_form = False

    # Ширина и высота окна
    width, height = 510, 400

    list_paging = True
    list_readonly = False

    # Значение колонки по-умолчанию, которое будет подбираться
    # при выборе значения из справочника
    column_name_on_select = 'name'

    # Добавлена ли возможность копирования
    allow_copy = False

    # права доступа для базовых справочников
    PERM_EDIT = 'edit'
    sub_permissions = {PERM_EDIT: u'Редактирование справочника'}

    def __init__(self):
        super(BaseDictionaryActions, self).__init__()
        # В отличие от обычных паков в этом экшены
        # создаются самостоятельно, а не контроллером
        # Чтобы было удобно обращаться к ним по имени
        self.list_window_action = DictListWindowAction()
        self.select_window_action = DictSelectWindowAction()
        self.edit_window_action = DictEditWindowAction()
        self.rows_action = DictRowsAction()
        self.last_used_action = DictLastUsedAction()
        self.row_action = ListGetRowAction()
        self.save_action = DictSaveAction()
        self.delete_action = ListDeleteRowAction()
        self.copy_action = DictCopyAction()
        # Но привязать их все равно нужно
        self.actions = [
            self.list_window_action,
            self.select_window_action,
            self.edit_window_action,
            self.rows_action,
            self.last_used_action,
            self.row_action,
            self.save_action,
            self.delete_action,
            self.copy_action
        ]

        # Исключение перехватываемое в экшенах, если объект не найден
        self._nofound_exception = ObjectNotFound

    #==================== ФУНКЦИИ ВОЗВРАЩАЮЩИЕ АДРЕСА =====================
    def get_list_url(self):
        """
        Возвращает адрес формы списка элементов справочника.
        Используется для присвоения адресов в прикладном приложении.
        """
        return self.list_window_action.get_absolute_url()

    #ISelectablePack
    def get_select_url(self):
        """
        Возвращает адрес формы списка элементов справочника.
        Используется для присвоения адресов в прикладном приложении.
        """
        return self.select_window_action.get_absolute_url()

    #ISelectablePack
    def get_edit_url(self):
        """
        Возвращает адрес формы редактирования элемента справочника.
        """
        return self.edit_window_action.get_absolute_url()

    def get_rows_url(self):
        """
        Возвращает адрес по которому запрашиваются элементы грида
        """
        return self.rows_action.get_absolute_url()

    #ISelectablePack
    def get_autocomplete_url(self):
        """
        Возвращает адрес по которому запрашиваются элементы
        подходящие введенному в поле тексту
        """
        return self.get_rows_url()

    #==================== ФУНКЦИИ ВОЗВРАЩАЮЩИЕ ДАННЫЕ =====================
    def get_rows(self, offset, limit, filter, user_sort=''):
        """
        Метод который возвращает записи грида в виде
        обычного питоновского списка.
        """
        raise NotImplementedError()

    def get_row(self, id):
        """
        Метод, который возвращает запись справочника
        с указанным идентификатором.
        """
        raise NotImplementedError()

    def get_last_used(self):
        """
        Метод, который возвращает список записей справочника,
        которые были выбраны конкретным пользователем в последнее время.
        Записи возвращаются в виде обычного питоновского списка.
        """
        raise NotImplementedError()

    def validate_row(self, obj, request):
        """
        Метод отвечает за проверку корректности полей сохраняемого объекта.
        Если все в порядке,
        то метод не возвращает ничего, иначе результат будет возвращен экшену.
        Т.е. вернуть можно любой из поддерживаемых в results.py объектов.
        """
        pass

    def save_row(self, obj):
        """
        Метод, который выполняет сохранение записи справочника.
        На момент запуска метода
        в параметре object находится именно та запись справочника,
        которую необходимо сохранить.
        """
        raise NotImplementedError()

    def delete_row(self, obj):
        """
        Метод, который выполняет удаление записи справочника.
        На момент запуска метода в
        параметре object находится именно та запись справочника,
        которую необходимо удалить.
        """
        raise NotImplementedError()

    #ISelectablePack
    def get_display_text(self, key, attr_name=None):
        """
        Получить отображаемое значение записи
        (или атрибута attr_name) по ключу key
        """
        row = self.get_row(key)
        if row is not None:
            name = attr_name if attr_name else self.column_name_on_select
            text = getattr(row, name)
            # getattr может возвращать метод, например verbose_name
            if callable(text):
                return text()
            else:
                return text

    #ISelectablePack
    def get_record(self, key):
        """
        Получить значение записи по ключу key
        """
        return self.get_row(key)

    #====================== РАБОТА С ОКНАМИ ===============================
    def get_list_window(self, win):
        """
        Возвращает настроенное окно типа "Список" справочника
        """
        return win

    def get_select_window(self, win):
        """
        Возвращает настроенное окно выбора из справочника
        """
        return win

    def get_edit_window(self, win):
        ''' Возвращает настроенное окно редактирования элемента справочника '''
        return win


class BaseDictionaryModelActions(BaseDictionaryActions):
    """
    Класс, который реализует действия со справочником,
    записи которого являются моделями.
    """
    # Настройки вида справочника (задаются конечным разработчиком)
    model = None
    # Список полей модели по которым будет идти поиск
    filter_fields = []

    # Порядок сортировки элементов списка. Работает следующим образом:
    # 1. Если в list_columns модели списка есть поле
    # code, то устанавливается сортировка по возрастанию этого поля;
    # 2. Если в list_columns модели списка нет поля code, но
    # есть поле name, то устанавливается сортировка по возрастанию поля name;
    # Пример list_sort_order = ['code', '-name']
    list_sort_order = None

    def __init__(self):
        super(BaseDictionaryModelActions, self).__init__()
        if self.model:
            self._nofound_exception = self.model.DoesNotExist

    def get_rows_modified(
            self, offset, limit, filter,
            user_sort='', request=None, context=None):
        '''
        Возвращает данные для грида справочника
        '''
        sort_order = user_sort.split(',') if user_sort else self.list_sort_order
        filter_fields = self._default_filter()
        query = self.model.objects.all()
        query = utils.apply_sort_order(query, self.list_columns, sort_order)
        query = utils.apply_search_filter(query, filter, filter_fields)
        if (hasattr(self, 'modify_rows_query') and
                callable(self.modify_rows_query)):
            query = self.modify_rows_query(query, request, context)
        total = query.count()
        if limit > 0:
            query = query[offset: offset + limit]
        result = {'rows': list(query), 'total': total}
        return result

    def get_rows(self, offset, limit, filter, user_sort=''):
        sort_order = user_sort.split(',') if user_sort else self.list_sort_order
        filter_fields = self._default_filter()
        query = utils.apply_sort_order(
            self.model.objects, self.list_columns, sort_order)
        query = utils.apply_search_filter(query, filter, filter_fields)
        total = query.count()
        if limit > 0:
            query = query[offset: offset + limit]
        result = {'rows': list(query.all()), 'total': total}
        return result

#    def modify_rows_query(self, query, request, context):
#        '''
#        Модифицирует запрос на получение данных.
#        Данный метод необходимо определить в
#        дочерних классах.
#        '''
#        return query

    def get_row(self, id):
        assert isinstance(id, int)
        # Если id нет, значит нужно создать новый объект
        if id == 0:
            record = self.model()
        else:
            record = self.model.objects.get(id=id)
        return record

    @transaction.commit_on_success
    def save_row(self, obj):
        obj.save()
        return OperationResult(success=True)

    def delete_row(self, objs):
        # Такая реализация обусловлена тем,
        # что IntegrityError невозможно отловить
        # до завершения транзакции, и приходится оборачивать транзакцию.
        @transaction.commit_on_success
        def delete_row_in_transaction(self, objs):
            message = ''
            if len(objs) == 0:
                message = u'Элемент не существует в базе данных.'
            else:
                for obj in objs:
                    if (isinstance(obj, BaseObjectModel) or
                        (hasattr(obj, 'safe_delete') and
                        callable(obj.safe_delete))):
                        try:
                            obj.safe_delete()
                        except RelatedError, e:
                            message = e.args[0]
                    else:
                        if not safe_delete(obj):
                            message = (
                                u'Не удалось удалить элемент %s. '
                                u'Возможно на него есть ссылки.' % obj.id)
                            break
            return OperationResult.by_message(message)
        # Тут пытаемся поймать ошибку из транзакции.
        try:
            return delete_row_in_transaction(self, objs)
        except Exception, e:
            # Встроенный в Django IntegrityError
            # не генерируется. Кидаются исключения
            # специфичные для каждого драйвера БД.
            # Но по спецификации PEP 249 все они
            # называются IntegrityError
            if e.__class__.__name__ == 'IntegrityError':
                message = (
                    u'Не удалось удалить элемент. '
                    u'Возможно на него есть ссылки.')
                return OperationResult.by_message(message)
            else:
                # все левые ошибки выпускаем наверх
                raise

    def _default_filter(self):
        """
        Устанавливаем параметры поиска по умолчанию 'code' и 'name' в случае,
        если у модели есть такие поля
        """
        filter_fields = self.filter_fields[:]
        if not filter_fields:
            filter_fields.extend(
                [field.attname for field in self.model._meta.local_fields
                    if field.attname in ('code', 'name')])
        return filter_fields


class BaseEnumerateDictionary(BaseDictionaryActions):
    """
    Базовый экшен пак для построения справочников
    основанных на перечислениях, т.е.
    предопределенных неизменяемых наборах значений.
    """
    # Класс перечисление с которым работает справочник
    enumerate_class = None

    list_paging = False  # Значений как правило мало и они влезают в одну страницу грида
    list_readonly = True
    list_columns = [('code', 'Код', 15),
                    ('name', 'Наименование')]

    def get_rows(self, offset, limit, filter, user_sort=''):
        """
        Возвращает данные для грида справочника
        """
        assert self.enumerate_class is not None, (
            'Attribute enumerate_class is not defined.')
        data = []
        for k, v in self.enumerate_class.values.items():
            if filter and v.upper().find(filter.upper()) < 0:
                continue
            else:
                data.append({'id': k, 'code': k, 'name': v})

        result = {'rows': data, 'total': len(data)}
        return result

    def get_row(self, id):
        """
        Заглушка для работы биндинга. В случае с перечислениями
        сам id хранится в БД
        """
        assert isinstance(id, int)
        assert id in self.enumerate_class.keys(), (
            'Enumarate key "%s" is not'
            ' defined in %s' % (id, self.enumerate_class))
        return id

    #ISelectablePack
    def get_display_text(self, key, attr_name=None):
        """
        Получить отображаемое значение записи
        (или атрибута attr_name) по ключу key
        """
        row_id = self.get_row(key)
        text = self.enumerate_class.values.get(row_id, '')
        return text

    #ISelectablePack
    def get_record(self, key):
        return self.get_row(key)



#coding:utf-8
'''
Интерфейсы
++++++++++
'''

from abc import ABCMeta, abstractmethod


class ISelectablePack(object):
    """
    Интерфейс pack-классов для выбора значений из полей ExtDictSelectField
    """
    __metaclass__ = ABCMeta

    @abstractmethod
    def get_edit_url(self):
        """ Получить адрес для запроса диалога редактирования
        выбранного элемента """

    @abstractmethod
    def get_autocomplete_url(self):
        """ Получить адрес для запроса элементов подходящих введенному
        в поле тексту """

    @abstractmethod
    def get_select_url(self):
        """ Получить адрес для запроса диалога выбора элемента """

    @abstractmethod
    def get_display_text(self, key, attr_name=None):
        """ Получить отображаемое значение записи (или атрибута attr_name)
        по ключу key """


class IMultiSelectablePack(ISelectablePack):
    """
    Интерфейс pack-классов для множественного выбора значений из
        полей ExtMultiSelectField
    """
    @abstractmethod
    def get_multi_select_url(self):
        """ Получить адрес для запроса диалога множественного
        выбора элемента """

    @abstractmethod
    def get_display_dict(self, key, value_field='id', display_field='name'):
        """
        Получить список словарей, необходимый для представления выбранных
        значений ExtMultiSelectField
        Пример результата: [{'id':0,'name':u''},]
        """



#coding:utf-8
"""
Платформа разработки приложений ERP типа на python и django
"""

import copy
import datetime
import json
import decimal
import sys

from django.db import models as dj_models
from django.http import HttpResponseRedirect, HttpResponseServerError
from django.conf import settings
from django.contrib import auth
from django.utils import datetime_safe
from django.views.debug import ExceptionReporter

import actions

from actions import ApplicationLogicException

from actions.urls import get_app_urlpatterns


def date2str(date, template=None):
    """
    datetime.strftime глючит с годом < 1900
    типа обходной маневр (взято из django)
    WARNING from django:
    # This library does not support strftime's \"%s\" or \"%y\" format strings.
    # Allowed if there's an even number of \"%\"s because they are escaped.
    """
    return datetime_safe.new_datetime(date).strftime(
        template or settings.DATE_FORMAT or '%d.%m.%Y')


class AutoLogout(object):
    '''
    Отслеживает активность пользователей в системе.
    Если с последнего действия пользователя прошло времени
    больше чем INACTIVE_SESSION_LIFETIME,
    то он выводит пользователя из системы
    '''

    session_key = 'app_last_user_activity_time'

    def process_request(self, request):
        # Если проверка отключена
        if settings.INACTIVE_SESSION_LIFETIME == 0:
            return

        # У аутентифицированного пользователя проверяем таймаут,
        # а ананимусов сразу посылаем
        if request.user.is_authenticated():
            last_time = request.session.get(self.session_key, None)
            if last_time is not None:
                delta = datetime.datetime.now() - last_time
                if delta.seconds / 60 > settings.INACTIVE_SESSION_LIFETIME:
                    # После логаута сессия уже другая
                    # и присваивать время не нужно
                    auth.logout(request)
                    return

            # Записываем время последнего запроса
            request.session[self.session_key] = datetime.datetime.now()


class M3JSONEncoder(json.JSONEncoder):
    def __init__(self, *args, **kwargs):
        self.dict_list = kwargs.pop('dict_list', None)
        super(M3JSONEncoder, self).__init__(*args, **kwargs)

    def default(self, obj):
        # обработаем простейшие объекты,
        # которые не обрабатываются стандартным способом
        if isinstance(obj, datetime.datetime):
            return '%02d.%02d.%04d %02d:%02d:%02d' % (
                obj.day, obj.month, obj.year, obj.hour, obj.minute, obj.second)
        elif isinstance(obj, datetime.date):
            return '%02d.%02d.%04d' % (obj.day, obj.month, obj.year)
        elif isinstance(obj, datetime.time):
            return obj.strftime('%H:%M')
        elif isinstance(obj, decimal.Decimal):
            return str(obj)

        # тут элементы, имеющие атрибут _config (Ext-контролы, обычно)
        # возвращаются в виде собственно конфига
        try:
            return obj._config
        except AttributeError:
            pass

        # Прошерстим методы и свойства, найдем те,
        # которые могут передаваться на клиента
        # Клонирование словаря происходит потому,
        # что сериализуемые методы переопределяются результатами своей работы
        cleaned_dict = {}
        dict = copy.copy(obj.__dict__)

        # Для джанговских моделей функция dir
        # дополнительно возвращает "ссылки" на связанные модели.
        # Их не нужно сериализовать, а также при обращении к ним
        # происходят запросы к БД. Причем на практике есть случаи,
        # когда эти запросы вызвают эксепешны
        # (например, если изменен id'шник объекта)
        related_objs_attrs = []
        manager_names = []
        if isinstance(obj, dj_models.Model):
            related_objs = obj._meta.get_all_related_objects()
            related_objs_attrs = [ro.var_name for ro in related_objs]
            # Также соберем все атрибуты-менеджеры (их может быть несколько).
            # Сюда попадет "objects", который исключаем из обработки ниже.
            for attr in obj.__class__.__dict__:
                if isinstance(
                    obj.__class__.__dict__[attr],
                    dj_models.manager.ManagerDescriptor
                ):
                    manager_names.append(attr)

        # если передали специальный список атрибутов, то пройдемся по ним
        # атрибуты вложенных объектов разделены точкой
        # будут созданы вложенные объекты для кодирования
        if self.dict_list:
            for item in self.dict_list:
                lst = item.split('.')
                value = obj
                arr = dict
                last_attr = None
                set_value = False
                for attr in lst:
                    if last_attr:
                        if last_attr not in arr:
                            arr[last_attr] = {}
                        else:
                            if not isinstance(arr[last_attr], type({})):
                                value = None
                                # у объекта уже стоит свойство не словарь,
                                # видимо оно пришло откуда-то свыше
                                set_value = False
                                break
                        arr = arr[last_attr]
                    if hasattr(value, attr):
                        value = getattr(value, attr)
                        if callable(value):
                            # это не свойство а функция, вызовем ее
                            value = value()
                        # нашли свойство, значит его надо будет
                        # поставить после цикла
                        set_value = True
                    else:
                        value = None
                    last_attr = attr
                if set_value:
                    arr[attr] = value

        for attr in dir(obj):
            # Во всех экземплярах моделей Django есть атрибут "objects",
            # т.к. он является статик-атрибутом модели.
            # Но заботливые разработчики джанги позаботились о нас
            # и выкидывают спицифичную ошибку
            # "Manager isn't accessible via %s instances"
            # при обращении из экземпляра.
            # Поэтому "objects" нужно игнорировать.
            # Да и вообще все менеджеры надо игнорировать,
            # т.к. их имена мы собираем выше.
            # Также проигнорируем приватные и протектнутные атрибуты
            if (
                not attr.startswith('_')
                and attr not in manager_names
                and attr != 'tree'
                and attr not in related_objs_attrs
            ):
                try:
                    # если метод или свойство есть в классе,
                    # то проверим у него признак
                    class_attr_value = getattr(obj.__class__, attr, None)
                    if not class_attr_value is None:
                        json_encode = getattr(
                            class_attr_value, 'json_encode', False)
                        if json_encode:
                            value = getattr(obj, attr)
                            if callable(value):
                                # если это метод, то вызовем его
                                dict[attr] = value()
                            else:
                                # иначе это было свойство или какой-то атрибут
                                dict[attr] = value
                except Exception, exc:
                    # Вторая проблема с моделями в том,
                    # что dir кроме фактических полей возвращает ассессоры.
                    # При попытке обратиться к ним происходит запрос(!)
                    # и может возникнуть ошибка DoesNotExist
                    # Заботливые разработчики Django
                    # сделали её разной для всех моделей ;)
                    if exc.__class__.__name__.find('DoesNotExist') == -1:
                        raise

        for attribute in dict.keys():
            # Для полей типа myfield_id автоматически создается атрибут,
            # ссылающияся на наименование, например,
            # для myfield_id будет myfield_ref_name,
            # конечно если у модели myfield есть name.
            # Зачем это нужно - х.з.
            if len(attribute) > 3 and attribute.endswith('_id'):
                try:
                    field_name = attribute[0:len(attribute) - 3]
                    if getattr(getattr(obj, field_name), 'name'):
                        if callable(getattr(getattr(obj, field_name), 'name')):
                            cleaned_dict[field_name + '_ref_name'] = getattr(
                                getattr(obj, field_name), 'name')()
                        else:
                            cleaned_dict[field_name + '_ref_name'] = getattr(
                                getattr(obj, field_name), 'name')
                except:
                    pass
            if len(attribute) > 6 and attribute.endswith('_cache'):
                # вережим этот кусок, т.к. если есть кэш на ForeignKey,
                # то он отработался на верхнем этапе
                # а если кэш на что-то другое (set etc),
                # то фиг знает какое свойство у него надо брать
                pass
            # Ибо нефиг сериализовать protected/private атрибуты!
            if attribute.startswith('_'):
                pass
            else:
                # просто передадим значение,
                # оно будет закодировано в дальнейшем
                cleaned_dict[attribute] = dict[attribute]
        return cleaned_dict


def json_encode(f):
    """
    Декоратор, которым нужно отмечать сериализуемые в M3JSONEncoder методы
    """
    f.json_encode = True
    return f


class property_json_encode(property):
    """
    Декоратор для свойств, которые нужно отмечать сериализуемые в M3JSONEncoder
    """
    json_encode = True


class RelatedError(Exception):
    """
    Исключение для получения связанных объектов
    """
    pass


def authenticated_user_required(f):
    """
    Декоратор проверки того, что к обращение к требуемому ресурсу системы
    производится аутентифицированным пользователем
    """

    def action(request, *args, **kwargs):
        user = request.user
        if not user or not user.is_authenticated():
            if request.is_ajax():
                res = actions.OperationResult.by_message(
                    u'Вы не авторизованы. Возможно, закончилось время '
                    u'пользовательской сессии.<br>'
                    u'Для повторной аутентификации обновите страницу.')
                return res.get_http_response()
            else:
                return HttpResponseRedirect('/')
        else:
            return f(request, *args, **kwargs)

    return action


class PrettyTracebackMiddleware(object):
    """
    Middleware, выводящая traceback'и в html-виде
    """
    def process_exception(self, request, exception):
        reporter = ExceptionReporter(request, *sys.exc_info())
        html = reporter.get_traceback_html()
        return HttpResponseServerError(html, mimetype='text/html')



#coding:utf-8
"""
Базовые классы экшенпаков для справочников нового стиля
"""

from simple import (BaseDictionaryActions,
                    BaseDictionaryModelActions)

from tree import (BaseTreeDictionaryActions,
                  BaseTreeDictionaryModelActions)


#coding: utf-8

import json
import hashlib
import logging
import time
import urllib2

from django.conf import settings
from django.contrib.auth.models import User
from django.contrib.sessions.models import Session
from django.db.models.signals import post_delete, post_save
from django.contrib.auth.signals import user_logged_in, user_logged_out

from m3.actions import ControllerCache

try:
    import pystatsd
except ImportError as ie:
    raise ImportError('Metrics collection is enabled, but we failed to '
                      'import "pystatsd": {0}'.format(unicode(ie)))

prefix = getattr(settings, 'METRICS_PREFIX', None)
host = getattr(settings, 'METRICS_HOST', 'localhost')
port = getattr(settings, 'METRICS_PORT', 8125)
endpoint_url = getattr(settings, 'METRICS_CONTEXTS_URL',
                       'http://localhost:1942/register')

statsd_client = pystatsd.Client(host, port, prefix=prefix)


def gauge_user_count():
    statsd_client.gauge('users.count', User.objects.count())


def gauge_active_users():
    statsd_client.gauge('users.active', Session.objects.count())


def logged_out(**kwargs):
    statsd_client.decr('users.active')


def logged_in(**kwargs):
    statsd_client.incr('users.active')


def user_deleted(**kwargs):
    statsd_client.decr('users.count')


def user_created(**kwargs):
    if not kwargs['instance'].id:
        statsd_client.incr('users.count')


def capture_users_metrics(statsd_client):
    gauge_user_count()
    gauge_active_users()

    if not getattr(User, '__metrics_registered', False):
        post_delete.connect(sender=User, receiver=user_deleted)
        post_save.connect(sender=User, receiver=user_created)

        user_logged_in.connect(receiver=logged_in)
        user_logged_out.connect(receiver=logged_out)

        User.__metrics_registered = True


def send_controllers_contexts():
    def get_hash(s):
        h = hashlib.md5()
        h.update(s)
        return h.hexdigest()

    assert prefix, 'Identity prefix (METRICS_PREFIX) should be specified!'

    ControllerCache.populate()
    controllers = {}    
    
    controller_urls = sorted([c.url for c in ControllerCache._controllers])
    for url in controller_urls:
        controllers[get_hash(url)] = dict(url=url)

    all_urls = [url for c in ControllerCache._controllers for url in c._url_patterns.keys()]
    urls = dict([(get_hash(u), u) for u in all_urls])

    parts = prefix.split('.')
    identity = dict(zip(['version', 'region', 'client', 'product'], parts))
    instance_id = [part for part in parts if part.startswith('instance_')]

    if instance_id:
        identity['instance'] = instance_id[0][9:]

    packet = {
        'timestamp': int(time.time()),
        'info': {
            'controller': controllers,
            'url': urls
        },
        'identity': identity
    }

    try:
        req = urllib2.Request(endpoint_url,
                              data=json.dumps(packet),
                              headers={'Content-Type': 'application/json'})
        urllib2.urlopen(req)
        logging.info('Successfully sent context information.')
    except urllib2.URLError as ue:
        logging.error("Can't send contexts to {0}: {1}".format(endpoint_url, ue))



#coding: utf-8
"""
Результаты выполнения экшенов
+++++++++++++++++++++++++++++
"""

import json
import abc
from copy import copy

from django import http
from django.conf import settings

from context import ActionContext


class ActionResult(object):
    __metaclass__ = abc.ABCMeta
    """
    Класс описывает результат выполнения Action'а
    Данный класс является абстрактным.
    """

    def __init__(self, data=None, http_params={}):
        """
        :param data: данные, на основе которых будет сформирован
            результат выполнения действия.
        :type data: Тип объекта, передаваемого через data
            зависит от дочернего результата.

        :param dict http_params: параметры http,
            которые будут помещены в ответ сервера
        """
        self.data = data
        self.http_params = (http_params is None and {}) or http_params

    @abc.abstractmethod
    def get_http_response(self):
        """
        :return: соответствующий данному
            результату выполнения действия ответ
        :rtype: django.http.HttpResponse
        """
        pass

    def process_http_params(self, response):
        """
        Добавляет параметры http в ответ

        :param response: ответ, в который добавляются параметры
        :type response: наследник m3_core.actions.results.ActionResult

        :return: http-ответ с добавленными параметрами
        :rtype: наследник m3_core.actions.results.ActionResult
        """
        if self.http_params:
            for k, v in self.http_params.items():
                response[k] = v
        return response


class PreJsonResult(ActionResult):
    """
    Результат выполнения операции в виде, например, списка объектов, готовых к
    сериализации в JSON формат и отправке в HttpResponse.
    В *data* передается объект для сериализации.
    В *dict_list* указывается список объектов
    и/или атрибутов вложенных объектов для
    более глубокой сериализации. Смотри класс т3.core.json.M3JSONEncoder.
    Параметр специфичный для проекта *secret_values*
    - используется чтобы указать,
    что передаются персональные обезличенные данные
    и их расшифровать перед отправкой клиенту.
    """

    def __init__(self, data=None, secret_values=False, dict_list=None):
        super(PreJsonResult, self).__init__(data)
        from m3 import M3JSONEncoder

        self.encoder_clz = M3JSONEncoder
        self.secret_values = secret_values
        self.dict_list = dict_list

    def get_http_response(self):
        encoder = self.encoder_clz(dict_list=self.dict_list)
        if settings.DEBUG:
            encoder.indent = 4
            encoder.separators = (',', ': ')
            encoder.sort_keys = True

        result = encoder.encode(self.data)
        response = http.HttpResponse(result, mimetype='application/json')
        if self.secret_values:
            response['secret_values'] = True
        return response


class JsonResult(ActionResult):
    """
    Результат выполнения операции
    в виде готового JSON объекта для возврата в response.
    Для данного класса в *data* храниться строка с данными JSON объекта.
    """

    def get_http_response(self):
        return http.HttpResponse(self.data, mimetype='application/json')


class HttpReadyResult(ActionResult):
    """
    Результат выполнения операции в виде готового HttpResponse.
    Для данного класса в *data* храниться объект класса HttpResponse.
    """

    def get_http_response(self):
        return self.data


class TextResult(ActionResult):
    """
    Результат, данные *data* которого напрямую передаются в HttpResponse
    """

    def get_http_response(self):
        return http.HttpResponse(self.data)


class XMLResult(ActionResult):
    """
    Результат в формате xml, данные которого напрямую передаются в HttpResponse
    """

    def get_http_response(self):
        return http.HttpResponse(
            self.data,
            mimetype='application/xml',
            content_type='text/xml; charset=utf-8'
        )


#==============================================================================
# Результаты выполнения операции с заданным контекстом
#==============================================================================
class BaseContextedResult(ActionResult):
    """
    Абстрактный базовый класс, который оперирует понятием результата
    выполнения операции, 'отягощенного некоторым контектом'
    """

    def __init__(self, data=None, context=None, http_params={}):
        super(BaseContextedResult, self).__init__(data, http_params)
        self.set_context(context)

    def set_context(self, context):
        if isinstance(context, ActionContext):
            # в случае если задан реальный
            # результат выполнения операции,
            # то его и регистрируем
            self.context = context
        elif context is None:
            # Пустой контекст
            self.context = ActionContext()
        else:
            raise TypeError('context must be a ActionContext instance')


class OperationResult(ActionResult):
    """
    Результат выполнения операции,
    описанный в виде Ajax результата ExtJS: success или failure.
    В случае если операция выполнена успешно,
    параметр *success* должен быть True, иначе False.

    :param boolean success: флаг успеха операции

    :param unicode message: сообщение, поясняющее
        результат выполнения операции.

    :param unicode code: текст javascript, который будет
        выполнен на клиенте в результате
        обработки результата операции.
    """

    def __init__(
            self, success=True, code='', message='', *args, **kwargs):
        super(OperationResult, self).__init__(*args, **kwargs)
        # Результат выполнения операции: успех/неудача
        self.success = success
        # Сообщение об ошибке выводимое при неудаче
        self.message = message
        # Произвольный JS код, который выполняется в любом случае если задан
        self.code = code

    @staticmethod
    def by_message(message):
        """
        Возвращает экземпляр OperationResult
        построенный исходя из сообщения *message*.
        Если сообщение не пустое,
        то операция считается проваленной и success=False,
        иначе операция считается успешной success=True.

        :param unicode message: текст сообщения об ошибке, или не указан
        """
        result = OperationResult()
        if message:
            result.success = False
            result.message = message
        return result

    def get_http_response(self):
        """
        Возвращает объект HttpResponse,
        соответствующий данному результату выполнения операции

        :return: http-ответ, соответствующий данному результату
        :rtype: django.http.HttpResponse
        """
        result = {
            'message': self.message,
            'success': True if self.success else False
        }
        result = json.JSONEncoder().encode(result)

        # Вставляем функцию прямо в JSON без кодирования
        if self.code:
            self.code = self.code.strip()
            if self.code[len(self.code) - 2:len(self.code)] == '()':
                code = ' ,"code": %s' % self.code[:-2]
            else:
                code = ' ,"code": %s' % self.code
            result = result[:-1] + code + result[-1]

        repsonse = http.HttpResponse(result)
        repsonse = self.process_http_params(repsonse)
        return repsonse


class ActionRedirectResult(object):
    """
    Перенаправляет обработку запроса на другой экшен.
    Экшен предварительно находится с помощью метода
    ActionController.get_action_url()
    """

    def __init__(self, action, context=None):
        self.action = action
        self.context = context

    def prepare_request(self, request):
        if self.context:
            new_post = copy(request.POST)
            for k, v in new_post.__dict__.iteritems():
                new_post[k] = getattr(self.context, k, v)
            request.POST = new_post
            del request._request
        return request



#coding:utf-8
'''
Различные модели кеширования информации в рантайме приложения

@author: akvarats
'''

import threading

class CacheStat(object):
    '''
    Класс, хранящий статистику обращения к кешу
    '''
    def __init__(self):
        self.in_cache = 0  # число попаданий в закешированные данные
        self.out_cache = 0 # число непопаданий в закешированные данные
        self.drops = 0 # количество сбросов кеша с момента создания
        self.full_drops = 0 # количество полных сбросов кеша с момента создания

class RuntimeCacheMetaclass(type):
    '''
    Метакласс, который позволяет делать синглтоны, объединенные в иерархию, но при этом
    каждый уровень иерархии имеет свой shared_state.

    Метакласс замещает __init__ управляемого класса, в результате чего этот метод
    недоступен прикладным разработчикам. Для реализации логики инициализации класса
    можно реализовать метод custom_init(*args, **kwargs).
    '''
    def __new__(cls, name, bases, attrs):

        def default_init(self, *args, **kwargs):
            self.__dict__ = self._shared_state
            # добавляем возможность вызова кастомного инита (обычный уже использован и
            # не может быть реализован в дочерних к RuntimeCache классах
            if hasattr(self, 'custom_init') and callable(self.custom_init):
                self.custom_init(*args, **kwargs)

            # в случае, если внутри класса кеша задан обработчик, то мы пытаемся его зарегистировать
            if hasattr(self, 'handler') and callable(self.handler) and not self.is_instance_registered():
                self.register_handler(self.handler)

        klass = super(RuntimeCacheMetaclass, cls).__new__(cls, name, bases, attrs)

        klass._shared_state = dict(
            handlers = {}, # словарь Хэндлеров, с ключом в виде имени класса Хэндлера
            handler_run_rules = {},
            data = {}, # собственно те данные, которые лежат в кеше
            write_lock = threading.RLock(),
            stat = CacheStat(),
        )
        klass.__init__ = default_init

        return klass

class RuntimeCache(object):
    '''
    Класс, используемый для кеширования данных в рантайме приложения.

    Использование данного класса:
    '''
    __metaclass__ = RuntimeCacheMetaclass

    def register_handler(self, handler):
        '''
        Регистрирует обработчик заполнения кешированных данных.
        '''
        # TODO: в качестве возможности будущего расширения функционала данного класса
        # можно предусмотреть возможность указания хендлеров сборки кеша
        # для отдельных значений измерений. Для хранения маппинга хендлеров
        # и измерений предусмотрен словарь handler_run_rules

        assert callable(handler), u'Обработчик заполнения кеша должен быть callable'

        # TODO: не совсем понятно, нужно ли ставить lock в данном случае
        try:
            self.write_lock.acquire()
            self.handlers[self.__class__.__name__] = handler
        finally:
            self.write_lock.release()

    def is_instance_registered(self):
        '''
        Проверяет, если ли в списке обработчиков кеша указанный хендлер
        '''
        return self.handlers.has_key(self.__class__.__name__)

    def _normalize_dimensions(self, dimensions):
        '''
        Выполняет нормализацию переданных измерений
        '''
        result = ()
        if dimensions:
            result = dimensions if isinstance(dimensions, tuple) else (dimensions,)
        return result

    def _need_populate(self, cleaned_dims):
        '''
        Проверяет, нужно ли выполнять прогрузку кеша для указанных измерений
        '''
        return not self.data.has_key(cleaned_dims)

    def _populate(self, dimensions):
        '''
        Метод собирает информацию по кешируемым объектам.

        Возвращает True, если populate реально состоялся
        '''
        dims = self._normalize_dimensions(dimensions)

        if not self._need_populate(dims):
            return False

        try:
            self.write_lock.acquire()
            if not self._need_populate(dims):
                return False
            for handler in self.handlers.itervalues():
                prepared_data = handler(self, dims)
                if isinstance(prepared_data, dict):
                    for key,value in prepared_data.iteritems():
                        self.set(key, value)
        finally:
            self.write_lock.release()

        return True

    def set(self, dimensions, value):
        '''
        Устанавливает значение в кеше
        '''
        self.data[self._normalize_dimensions(dimensions)] = value

    def get(self, dimensions, default=None):
        '''
        Возвращает значение из кеша
        '''
        dims = self._normalize_dimensions(dimensions)
        if self._populate(dims):
            self.stat.out_cache += 1
        else:
            self.stat.in_cache += 1
        return self.data.get(dims, default)

    def has_data(self, dimensions):
        '''
        Возвращает True в случае, если в кеше есть данные для указанных
        измерений
        '''
        dims = self._normalize_dimensions(dimensions)
        return self.data.has_key(dims)

    def get_size(self):
        '''
        Возвращает количество объектов, помещенных в кеш
        '''
        return len(self.data)

    def drop(self, dimensions):
        '''
        Метод сброса кеша по измерению
        '''
        dims = self._normalize_dimensions(dimensions)

        try:
            self.write_lock.acquire()
            self.data.pop(dims, None)
            self.stat.full_drops += 1
        finally:
            self.write_lock.release()

    def drop_all(self):
        try:
            self.write_lock.acquire()
            self.data = {}
            self.stat.full_drops += 1
        finally:
            self.write_lock.release()

    def clear_stat(self):
        self.stat = CacheStat()

    def lock(self):
        '''
        Устанавливает блокировку на работу с внутренними данными
        '''
        self.write_lock.acquire()

    def unlock(self):
        '''
        Снимает блокировку на работу с внутренними данными
        '''
        self.write_lock.release()


class IntegralRuntimeCache(RuntimeCache):
    '''
    Кеш данных, которые собирает все данные однократно и не пересобирает их после этого.
    (пока не произойдет сброс кеша)
    '''
    def _need_populate(self, cleaned_dims):
        return not self.data


#===============================================================================
# Кеши, которые работают не как синглтоны. Их экземпляры имеют свое состояние.
# Данные кеши используются в процессе
#===============================================================================

class ObjectStorage(object):
    '''
    Класс кешей, объекты которых не являются синглтонами
    '''
    def __init__(self):
        self.data = {}
        self.handlers = []

        # в случае, если внутри класса кеша задан обработчик, то мы пытаемся его зарегистировать
        if hasattr(self, 'handler') and callable(self.handler) and not self.handler_registered(self.handler):
            self.register_handler(self.handler)

    def register_handler(self, handler):
        '''
        Регистрирует обработчик заполнения кешированных данных.
        '''
        # TODO: в качестве возможности будущего расширения функционала данного класса
        # можно предусмотреть возможность указания хендлеров сборки кеша
        # для отдельных значений измерений. Для хранения маппинга хендлеров
        # и измерений предусмотрен словарь handler_run_rules

        assert callable(handler), u'Обработчик заполнения кеша должен быть callable'

        # TODO: не совсем понятно, нужно ли ставить lock в данном случае
        if handler not in self.handlers:
            self.handlers.append(handler)

    def handler_registered(self, handler):
        '''
        Проверяет, если ли в списке обработчиков кеша указанный хендлер
        '''
        return handler in self.handlers

    def _normalize_dimensions(self, dimensions):
        '''
        Выполняет нормализацию переданных измерений
        '''
        result = ()
        if dimensions:
            result = dimensions if isinstance(dimensions, tuple) else (dimensions,)
        return result

    def _need_populate(self, cleaned_dims):
        '''
        Проверяет, нужно ли выполнять прогрузку кеша для указанных измерений
        '''
        return not self.data.has_key(cleaned_dims)

    def _populate(self, dimensions):
        '''
        Метод собирает информацию по кешируемым объектам.

        Возвращает True, если populate реально состоялся
        '''
        dims = self._normalize_dimensions(dimensions)

        if not self._need_populate(dims):
            return False

        if not self._need_populate(dims):
            return False
        for handler in self.handlers:
            prepared_data = handler(self, dims)
            if isinstance(prepared_data, dict):
                for key,value in prepared_data.iteritems():
                    self.set(key, value)


        return True

    def set(self, dimensions, value):
        '''
        Устанавливает значение в кеше
        '''
        self.data[self._normalize_dimensions(dimensions)] = value

    def get(self, dimensions, default=None):
        '''
        Возвращает значение из кеша
        '''
        dims = self._normalize_dimensions(dimensions)
        self._populate(dims)
        return self.data.get(dims, default)

    def has_data(self, dimensions):
        '''
        Возвращает True в случае, если в кеше есть данные для указанных
        измерений
        '''
        dims = self._normalize_dimensions(dimensions)
        return self.data.has_key(dims)

    def get_size(self):
        '''
        Возвращает количество объектов, помещенных в кеш
        '''
        return len(self.data)

    def drop(self, dimensions):
        '''
        Метод сброса кеша по измерению
        '''
        dims = self._normalize_dimensions(dimensions)

        self.data.pop(dims, None)

    def drop_all(self):
        self.data = {}

    def clear_stat(self):
        self.stat = CacheStat()


class IntegralObjectStorage(ObjectStorage):
    '''
    Кеш данных, которые собирает все данные однократно и не пересобирает их после этого.
    (пока не произойдет сброс кеша)
    '''
    def _need_populate(self, cleaned_dims):
        return not self.data


class ModelObjectStorage(IntegralObjectStorage):
    '''
    Хранилище объектов, которые считывает все записи в моделях указанных типов.

    Ключом данных является идентификатор объекта.
    '''
    def __init__(self, model=None):
        super(ModelObjectStorage, self).__init__()
        self.model = model

    def handler(self, cache, dimensions):
        '''
        Обработчик кеша. Выполняется чтение всех данных из указанной в конструкторе
        модели.
        '''
        self.drop_all()
        if self.model:
            objects = self.model.objects.all()
            for object in objects:
                self.set(object.id, object)


#===============================================================================
# Класс "Фабрика объектных хранилищ"
#===============================================================================
class ModelObjectStorageFactory(object):
    '''
    Класс, отвечающий за организацию прозрачного интерфейса доступа к
    объектным хранилищам по различным типам моделей.

    Класс является thread-unsafe
    '''
    def __init__(self):
        self.storages = {}

    def get_storage(self, model):
        '''
        Получение объекта класса ModelObjectStorage для указанного типа модели.

        @param model: класс моделей, для которых выполняется построение
                      объектного хранилища.

        В случае, если значение параметр model не является классом наследником
        django.db.Model, то выбрасывается исключение TypeError
        '''
        if (not model or
            not hasattr(model, 'objects')):
            raise TypeError(u'Для построения объектного хранилища ожидается класс-наследник django.db.Model')

        if self.storages.has_key(model):
            result = self.storages[model]
        else:
            result = ModelObjectStorage(model=model)
            self.storages[model] = result

        return result

    def drop(self):
        '''
        Очищает внутренние структуры фабрики
        '''
        for storage in self.storages.values():
            storage.drop_all()









#coding:utf-8
'''
Created on 25.10.2011

@author: akvarats
'''
from django import template

from m3.actions import urls

register = template.Library()

@register.simple_tag
def action_url(shortname):
    '''
    Темплейт таг, который возвращает URL экшена
    '''
    return urls.get_url(str(shortname))
    


# -*- coding: utf-8 -*-
"""
Модуль позволяющий реализовать управление порядком выполннения
обработчиков сигналов возбужденных в контексте некоей операции,
"завернутой" в транзакцию.

Типичный сценарий:

# сигнал на сохранение объекта пользователя
@receiver(post_save, sender=User)
@delay_in_situations('user_import', 'user_creation')
def save_user(model, obj, created, *args):
    if created:
        data = {'username': obj.username}
        def send_email():
            print "Sending EMail...."
            print data
    else:
        def send_mail():
            pass
    return send_email

...

# где-то в коде импорта пользователей
with TransactionCM('user_import'):
    for login, pwd in source:
        with SavePointCM():
            user = User(username=login)
            user.set_password(pwd)
            user.save()
"""

from uuid import uuid4

from django.db import transaction

from threading import local as _local


_state = _local()

def _reset_state():
    _state.rolled_back = set()
    _state.task_queue = []
    _state.cookies = []
    _state.situation = None

_reset_state()


class AbortTransaction(Exception):
    """
    Исключение, прерывающее текущий CM (SavePointCM/TransactionCM).
    Поймав это исключение CM откатывает изменения,
    произошедшие с момента входа в него и корректно завершается.
    """
    pass


class SavePointCM(object):
    """
    Context Manager создающий на время своего действия savepoint
    """

    def __init__(self):
        assert _state.situation, (
            u'Must be nested in TransactionCM!'
        )
        cookie = uuid4()
        _state.cookies.append(cookie)
        self._cookie = cookie
        self._sid = None

    def __enter__(self):
        self._sid = transaction.savepoint()
        return self

    def __exit__(self, ex_type, ex_inst, traceback):
        _state.cookies.remove(self._cookie)
        if ex_type is None:
            transaction.savepoint_commit(self._sid)
            return True
        else:
            transaction.savepoint_rollback(self._sid)
            _state.rolled_back.add(self._cookie)
            return (ex_type is AbortTransaction)

    def rollback(self):
        raise AbortTransaction()


class TransactionCM(object):
    """
    ContextManager создающий "ручную" транзакцию
    """
    def __init__(self, situation, rollback_all=True, using=None):
        assert not _state.situation, (
            u'Nested contexts not supported!'
        )
        _reset_state()
        _state.situation = self._situation = situation

        self._using = using

    def __enter__(self):
        transaction.enter_transaction_management(True, self._using)
        return self

    def __exit__(self, ex_type, ex_inst, traceback):
        if ex_type is None:
            transaction.commit(self._using)
            for (task, cookie) in _state.task_queue:
                if cookie not in _state.rolled_back:
                    task()
            _reset_state()
            return True
        else:
            transaction.rollback(self._using)
            _reset_state()
            return (ex_type is AbortTransaction)

    def rollback(self):
        raise AbortTransaction()


def delay_in_situations(*situations):
    """
    Оборачивает функцию и возвращает функцию-обработчик сигнала Django.
    В контексте одной из ситуаций, перечисленных в @situations
    выполнение функции, возвращённой оборачиваемой функцией,
    будет отложено до успешного завершения TransactionCM, создавшего ситуацию.
    """
    def wrapper(fn):
        def inner(*args, **kwargs):
            cont = fn(*args, **kwargs)
            assert callable(cont), "Task must return the callable object!"
            situation = _state.situation
            if not situation or situation not in situations:
                cont()
            else:
                cookie = (_state.cookies or [None])[-1]
                _state.task_queue.append((cont, cookie))
        return inner
    return wrapper



#coding:utf-8
'''
@author: pirogov
'''
import sys

from django.core.management.base import BaseCommand

from m3.actions import ControllerCache


class Command(BaseCommand):
    '''
    Запуск doctests для указанного модуля
    '''
    help = 'Run ControllerCache self-test'

    def handle(self, *args, **options):
        warns = ControllerCache._self_test()
        for warn in warns:
            sys.stderr.write('Self-test warning: %s\n' % warn)



#coding:utf-8
"""
Экшены для работы в асинхронном режиме
++++++++++++++++++++++++++++++++++++++
"""
__author__ = 'ZIgi'

from abc import ABCMeta, abstractmethod
from threading import Thread
import json

from django import http
from django.conf import settings
try:
    from django.utils.log import logger
except ImportError:
    from django.utils.log import getLogger
    logger = getLogger('django')

from m3.actions import Action, ACD
from m3.actions.results import ActionResult

# !!!!!
# для работы данного модуля необходимо наличие приложения 'm3_mutex'
# в INSTALLED_APPS приложения

# TODO: переделать по правильному - платформа не должна импортировать
# модули контрибов

if not 'm3_mutex' in settings.INSTALLED_APPS:
    # При сборке документации внешняя Django ничего не знает про m3_mutex
    logger.warning(
        u'For working async operations "m3_mutex" '
        u'must be define in INSTALLED_APPS')
    # raise ImportError(
    # u'For working async operations "m3_mutex" must'
    # ' be define in INSTALLED_APPS')

try:
    from m3_mutex import (
        capture_mutex, release_mutex, request_mutex,
        MutexID, MutexOwner, MutexBusy, MutexState, TimeoutAutoRelease)
except ImportError:
    logger.warning(u'm3_mutex import error')


class IBackgroundWorker(Thread):
    '''
    Абстрактный класс для исполнения кода в фоновом режиме.
    Соответсвующие методы должны быть определены разработчиком.
    Тк класс представляет собой наследник Thread, нужно понимать
    что фактически при использовании будет создан новый тред,
    и учитывать это при доступе к ресурсам приложения, кои могут
    быть модифицированы другими тредами. Методы stop, start и ping
    должны возвращать экземпляры  AsyncOperationResult.
    Вообще говоря, можно реализовать данный класс подругому,
    главное чтобы был имплементирован интерфейс
    (например если есть нужда использовать мутекс на уровне
    нескольких инстансов приложений)
    '''
    __metaclass__ = ABCMeta

    def __init__(self, boundary='', context=None, *args, **kwargs):
        super(IBackgroundWorker, self).__init__()
        self.boundary = boundary
        self.context = context

    def start(self):
        '''
        Запускает исполнение кода. Метод не абстрактный, при
        замещении обязательно вызывать суперкласс
        '''
        super(IBackgroundWorker, self).start()

    @abstractmethod
    def run(self):
        '''
        Метод вызывается после вызоыва метода start() в новом потоке,
        здесь фактически следует писать собственный код
        '''
        raise NotImplementedError(
            'Method run() must be overrided in  child class')

    @abstractmethod
    def stop(self):
        '''
        Метод остановки операции
        '''
        raise NotImplementedError(
            'Method stop() must be overrided in  child class')

    @abstractmethod
    def request(self):
        '''
        Запрос состояния операции
        '''
        raise NotImplementedError(
            'Method request() must be overrided in  child class')

    @abstractmethod
    def result(self):
        '''
        Запрос на получение результата асинхронной операции
        '''
        raise NotImplementedError(
            'Method result() must be overrided in  child class')

    # ------------------------------------------------------------------------
    # Методы работы c глобальными блокировками операции с указанным инстансом.
    # В базовом классе глобальная блокировка реализована с помощью семафоров
    # ------------------------------------------------------------------------
    MUTEX_GROUP = 'asyncop'
    RESULT_MUTEX_GROUP = 'asyncop-result'

    MUTEX_TIMEOUT = 600
    RESULT_MUTEX_TIMEOUT = 300

    def _mutex_id(self):
        return MutexID(group=IBackgroundWorker.MUTEX_GROUP, id=self.boundary)

    def _mutex_owner(self):
        return MutexOwner(
            name=IBackgroundWorker.MUTEX_GROUP, session_id=self.boundary)

    def _result_mutex_id(self):
        return MutexID(
            group=IBackgroundWorker.RESULT_MUTEX_GROUP, id=self.boundary)

    def _result_mutex_owner(self):
        return MutexOwner(
            name=IBackgroundWorker.RESULT_MUTEX_GROUP,
            session_id=self.boundary)

    def lock(self):
        '''
        Устанавливает глобальную блокировку по инстансу с использованием
        механизма мютексов
        '''
        try:
            capture_mutex(
                mutex_id=self._mutex_id(), owner=self._mutex_owner(),
                auto_release=TimeoutAutoRelease(timeout=self.MUTEX_TIMEOUT))
        except MutexBusy:
            raise Exception(
                u'asyncop: Не удалось установить'
                u' глобальную блокировку операции.')

    def unlock(self):
        '''
        Освобождает глобальную блокировку состояния
        '''
        release_mutex(mutex_id=self._mutex_id(), owner=self._mutex_owner())

    def check_state(self):
        '''
        Проверяет состояние глобальной блокировки операции.
        Возвращает кортеж из двух элементов (is_active, status_data),
        где is_active=True/False - показывает активность
        установленной блокировки, и status_data - произвольный
        объект (чаще строка), в котором
        находится описание состоания операции.
        '''
        mutex_state, mutex = request_mutex(self._mutex_id())
        return (
            (mutex_state != MutexState.FREE, mutex.status_data)
            if mutex is not None else '')

    def refresh_state(self, status_data):
        '''
        Обновляет состояние блокировки
        '''
        try:
            capture_mutex(
                mutex_id=self._mutex_id(), owner=self._mutex_owner(),
                status_data=status_data,
                auto_release=TimeoutAutoRelease(timeout=self.MUTEX_TIMEOUT))
        except MutexBusy:
            raise Exception(
                u'asyncop: Не удалось установить'
                u' глобальную блокировку операции.')

    def lock_result(self, result):
        '''
        Блокирует результат выполнения операции
        '''
        try:
            capture_mutex(
                mutex_id=self._result_mutex_id(),
                owner=self._result_mutex_owner(),
                auto_release=TimeoutAutoRelease(
                    timeout=self.RESULT_MUTEX_TIMEOUT),
                status_data=result or '')
        except MutexBusy:
            raise Exception(
                u'asyncop: Не удалось сохранить'
                u' результат выполнения оajyjперации.')

    def unlock_result(self):
        release_mutex(
            mutex_id=self._result_mutex_id(),
            owner=self._result_mutex_owner(),)

    def request_result(self):
        mutex_state, mutex = request_mutex(self._result_mutex_id())
        return (
            (mutex_state != MutexState.FREE, mutex.status_data)
            if mutex is not None else '')


class AsyncOperationResult(ActionResult):
    '''
    Результат выполнения асинхронной операции.
    '''

    def __init__(self, value=0.0, text='', alive=True):
        self.value = value
        self.text = text
        self.alive = alive

    def get_http_response(self):
        data = json.dumps({
            'text': self.text,
            'value': self.value,
            'alive': self.alive,
        })
        return http.HttpResponse(data, mimetype='application/json')


class AsyncAction(Action):
    '''
    Экшен обработки запросов с клиента. В данном варианте инстанс
    класса воркера общий для всех http запросов экземпляра приложения,
    поэтому если один пользователь начал фоновую операцию, все пользователи
    этого сервера будут видеть такой же прогресс. Можно с помощью словаря
    сессий, где бы хранились инстансы воркеров, реализовать для каждого
    пользователя свою операцию, или же с помощью мутексов глобальную
    блокировку операции на уровне приложения. При наследовании должен
    быть определен атрибут worker_cls - класс
    наследующий/имплементирующий IBackgroundWorker
    '''
    COMMAND_START = 'start'
    COMMAND_STOP = 'stop'
    COMMAND_REQUEST = 'request'
    COMMAND_RESULT = 'result'

    worker_cls = None

    def __init__(self):
        super(AsyncAction, self).__init__()

    def context_declaration(self):
        return [
            ACD(name='command', type=str, required=True),
            ACD(name='boundary', type=str, required=True,
                default='default-boundary'), ]

    def _worker_instance(self, request, context, *args, **kwargs):
        '''
        Метод, возвращающий экземпляр воркера фоновой операции.
        Данный метод можно переопределять для случаев, когда в
        воркер необходимо передавать дополнительные параметры.
        '''
        return self.worker_cls(boundary=context.boundary, context=context)

    def run(self, request, context):

        if self.worker_cls is None:
            raise NotImplementedError('Worker class is not defined')

        if context.command == self.COMMAND_START:
            return self.start_operation(request, context)
        elif context.command == self.COMMAND_STOP:
            return self.stop_operation(request, context)
        elif context.command == self.COMMAND_REQUEST:
            return self.request_operation(request, context)
        elif context.command == self.COMMAND_RESULT:
            return self.result_operation(request, context)

    def start_operation(self, request, context):
        worker_instance = self._worker_instance(request, context)
        # если операция уже запущенна, то перезапустим
        # надо подумать насколько корректно такое поведение
        worker_instance.stop()

        return worker_instance.start()

    def stop_operation(self, request, context):
        return self._worker_instance(request, context).stop()

    def request_operation(self, request, context):
        return self._worker_instance(request, context).request()

    def result_operation(self, request, context):
        return self._worker_instance(request, context).result()



#coding: utf-8
"""
Хелперы для отработки расширяемых конфигураций url'ов
+++++++++++++++++++++++++++++++++++++++++++++++++++++
"""

import collections
import inspect

import warnings

from django.conf import settings
from django.utils import importlib
from django.conf import urls


from m3 import caching

from m3.actions import ControllerCache, Action


def _get_instance(obj):
    if inspect.isclass(obj):
        warnings.warn(
            message=(
                "Возможность использования классов экшнов/паков"
                " вместо экземпляров, будет ликвидирована!"
            ),
            category=FutureWarning,
            stacklevel=2,
        )
        return obj()
    return obj


def get_app_urlpatterns():
    '''
    Возвращает конфигурацию урлов, объявленных в app_meta приложений.

    Данная функция не проглатывает ошибки, а выбрасывает все наружу.
    Перехват исключительных ситуаций данной фунции необходимо осуществлять
    вручную в urls.py прикладных приложений
    '''
    url_patterns = urls.defaults.patterns('',)

    for app_name in settings.INSTALLED_APPS:
        try:
            module = importlib.import_module('.app_meta', app_name)
        except ImportError, err:
            # по идее, такая ошибка возникает в
            # случае, если у нас для
            # установленного приложения
            # нет модуля app_meta.py
            if err.args[0].find('No module named') == -1:
                raise
            continue
        proc = getattr(module, 'register_urlpatterns', None)
        if callable(proc):
            url_patterns += proc()

    return url_patterns


def get_pack(pack_name):
    '''
    Получает экшенпак по имени
    '''
    pack_data = PacksNameCache().get(pack_name, None)
    return pack_data[0] if pack_data else None


def get_pack_instance(pack_name):
    '''
    Получает экземпляр набора экшенов по имени из контроллеров
    '''
    pack_data = PacksNameCache().get(pack_name, None)
    return pack_data[2] if pack_data else None


def get_action(action_name):
    '''
    Возвращает полный класс экшена, объявленного с указанным
    квалифицирующим именем.
    '''
    action_data = ActionsNameCache().get(action_name, None)
    return action_data[0] if action_data else None


def get_url(action):
    """
    Возвращает абсолютный путь до
    """
    names = []
    if isinstance(action, Action):
        # гениальных ход! - с чем пришли, то и ищем :)
        names.append(action.get_absolute_url())
    elif inspect.isclass(action) and issubclass(action, Action):
        names.append("%s.%s" % (action.__module__, action.__name__))
    elif isinstance(action, str):
        names.append(action)

    action_data = None
    for name in names:
        action_data = ActionsNameCache().get(name, None)
        if action_data:
            break
    return action_data[1] if action_data else ''

get_acton_url = get_url


def get_pack_url(pack_name):
    '''
    Возвращает абсолютный путь для набора экшенов
    '''
    pack_data = PacksNameCache().get(pack_name, None)
    return pack_data[1] if pack_data else ''


def get_pack_by_url(url):
    '''
    Возвращает набор экшенов по переданному url
    '''
    ControllerCache.populate()
    packs = collections.deque([])
    for controller in ControllerCache.get_controllers():
        packs.extend(controller.top_level_packs)

    while len(packs) > 0:
        pack = packs.popleft()
        if hasattr(pack, 'subpacks'):
            packs.extend(pack.subpacks)

        cleaned_pack = _get_instance(pack)
        if url == cleaned_pack.__class__.absolute_url():
            return cleaned_pack
    return None


#==============================================================================
# Кеш, используемый для хранения соответствия экшенов
#==============================================================================
class ActionsNameCache(caching.IntegralRuntimeCache):
    '''
    Кеш, используемый для хранения соответствия имен экшенов и паков
    соответствующим пакам
    '''

    def handler(self, cache, dimentions):
        '''
        Хендлер сборки кеша
        '''
        return inner_name_cache_handler(for_actions=True)


class PacksNameCache(caching.IntegralRuntimeCache):
    '''
    Кеш, используемый для хранения соответствия имен экшенов и паков
    соответствующим пакам
    '''

    def handler(self, cache, dimentions):
        '''
        Хендлер сборки кеша
        '''
        return inner_name_cache_handler(for_actions=False)


def inner_name_cache_handler(for_actions=True):
    '''
    Внутренний метод обхода дерева паков и экшенов.
    Используется в хендлерах сборки кешей
    '''
    def get_shortname(obj):
        '''
        Возвращает короткое имя для экшена или пака.
        Сам объект экшена или пака передается в параметре
        obj.
        '''
        names = ['shortname', 'short_name', ]
        objects = [obj.__class__, obj]
        for o in objects:
            for name in names:
                if hasattr(o, name) and isinstance(getattr(o, name), str):
                    return getattr(o, name, '')
        return ''

    # TODO посмотреть как работает для врапнутых классов
    result = {}

    # fullpaths - словарь, который хранит соответствие объекта (контроллера,
    # пака или экшена) и полного пути до него по шортнеймам
    fullpaths = {}

    ControllerCache.populate()
    # что-то внутренность данного метода не вызывает
    # доверия, если честно

    packs = collections.deque([])

    controllers = ControllerCache.get_controllers()

    # считываем паки верхнего уровня
    for controller in controllers:
        packs.extend(controller.top_level_packs)

        # добавляем полные пути в fullpaths
        fullpaths[controller] = get_shortname(controller)
        for pack in controller.top_level_packs:
            fullpaths[pack] = '%s.%s' % (
                fullpaths.get(controller, ''),
                get_shortname(pack)
            )

    while len(packs) > 0:
        pack = packs.popleft()
        # субпаки - в очередь!
        if hasattr(pack, 'subpacks'):
            packs.extend(pack.subpacks)

            for subpack in pack.subpacks:
                fullpaths[subpack] = '%s.%s' % (
                    fullpaths.get(pack, ''),
                    get_shortname(subpack),
                )

        if for_actions and hasattr(pack, 'actions'):
            for action in pack.actions:
                keys = []
                cache_object = None
                # если имеем дело с экземпляром
                # экшена, то ключем будет его
                # полный url
                if isinstance(action, Action):
                    cleaned_action = action
                    url = cleaned_action.get_absolute_url()
                    long_class_name = (
                        cleaned_action.__class__.__module__ + '.' +
                        cleaned_action.__class__.__name__
                    )
                    # регистрируем для url и для класса
                    cache_object = (
                        cleaned_action.__class__, url, cleaned_action)
                    keys.extend([url, long_class_name])
                else:
                    # неважно что нам передали, нам нужен экземпляр класса
                    cleaned_action = _get_instance(action)
                    # TODO: здесь url может быть не
                    # правильный, т.к. мы сами создали
                    # экземпляр и у него нет ни Pack, ни
                    # Controller.
                    # TODO: поэтому берем его через
                    # absolute_url, т.к. он ищет экземпляр
                    # экшена во всех контроллерах
                    # TODO: а это работает только для единичных экземпляров
                    url = cleaned_action.__class__.absolute_url()
                    long_class_name = (
                        cleaned_action.__class__.__module__ + '.' +
                        cleaned_action.__class__.__name__
                    )
                    # регистрируем для класса
                    cache_object = (
                        cleaned_action.__class__, url, cleaned_action)
                    keys.append(long_class_name)

                # регистрируем для shortname
                shortname = get_shortname(cleaned_action)
                if shortname:
                    keys.append(shortname)
                    keys.append('%s.%s' % (fullpaths.get(pack, ''), shortname))

                # регистрируем
                for key in keys:
                    result[key] = cache_object
        else:
            cleaned_pack = _get_instance(pack)
            url = cleaned_pack.__class__.absolute_url()
            cache_object = (cleaned_pack.__class__, url, cleaned_pack)
            # регистрируем как полный класс с модулем, так и просто имя класса
            keys = [
                '%s.%s' % (
                    cleaned_pack.__class__.__module__,
                    cleaned_pack.__class__.__name__
                ),
                cleaned_pack.__class__.__name__
            ]
            # регистрируем shortname
            shortname = get_shortname(cleaned_pack)
            if shortname:
                keys.append(shortname)
                if pack in fullpaths:
                    keys.append(fullpaths[pack])
            # регистрируем
            for key in keys:
                result[key] = cache_object

    return result



# -*- coding: utf-8 -*-
"""
Исключения, возникающие при работе контроллера
++++++++++++++++++++++++++++++++++++++++++++++
"""


class ApplicationLogicException(Exception):
    '''
    Исключительная ситуация уровня бизнес-логики приложения.
    '''

    def __init__(self, message):
        # Это исключение плохо подвергается pickle! пришлось дописать.
        # пруф:
        #   http://bugs.python.org/issue1692335
        #   http://bugs.python.org/issue13751
        Exception.__init__(self, message)
        self.exception_message = message

    def __str__(self):
        return self.exception_message


class ActionException(Exception):
    def __init__(self, clazz, *args, **kwargs):
        super(Exception, self).__init__(*args, **kwargs)
        self.clazz = clazz


class ActionNotFoundException(ActionException):
    """
    Возникает в случае, если экшен не найден ни в одном контроллере
    """
    def __str__(self):
        return 'Action "%s" not registered in controller/pack' % self.clazz


class ActionPackNotFoundException(ActionException):
    """
    Возникает в случае, если пак не найден ни в одном контроллере
    """
    def __str__(self):
        return 'ActionPack "%s" not registered in controller/pack' % self.clazz


class ReinitException(ActionException):
    """
    Возникает, если из-за неправильной структуры паков один и тот же
    экземпляр экшена может быть повторно инициализирован неверными значениями.
    """
    def __str__(self):
        return 'Trying to overwrite class "%s"' % self.clazz


class ActionUrlIsNotDefined(ActionException):
    """
    Возникает если в классе экшена не задан атрибут url.
    Это грозит тем, что контроллер не сможет найти и вызвать
    данный экшен при обработке запросов.
    """
    def __str__(self):
        return (
            'Attribute "url" is not defined '
            'or empty in action "%s"' % self.clazz
        )



# -*- coding: utf-8 -*-
"""
Static files finders
"""
import os
from importlib import import_module

from django.contrib.staticfiles.finders import AppDirectoriesFinder, BaseFinder
from django.core.files.storage import FileSystemStorage
from django.utils.datastructures import SortedDict
from django.conf import settings


class RecursiveAppDirectoriesFinder(AppDirectoriesFinder):
    """
    Поисковик файлов статики в подпапках приложений.
    Ищет в приложениях подпапки, содержащие папку static,
    но при этом не зарегистрированный, как приложения.
    Такие папки не подхватываются AppDirectoriesFinder'ом,
    но при этом для декомпозиции бывает полезно
    разнести статику внутри приложения

    ВАЖНО! Прописывать в settings его нужно после
    'django.contrib.staticfiles.finders.AppDirectoriesFinder'
    """
    storage_class = FileSystemStorage

    def __init__(self, apps=None, *args, **kwargs):
        self.apps = []
        self.storages = SortedDict()
        visited = set()

        def traverse(path, root=False):
            for sub in os.listdir(path):
                full_path = os.path.abspath(os.path.join(path, sub))
                if full_path not in visited and os.path.isdir(full_path):
                    if sub == 'static' and not root:
                        storage = self.storage_class(full_path)
                        storage.prefix = None  # Иначе не работает
                        self.apps.append(full_path)
                        self.storages[full_path] = storage
                    else:
                        traverse(full_path)

        for app in settings.INSTALLED_APPS:
            mod = import_module(app)
            location = os.path.dirname(mod.__file__)
            traverse(location, root=True)

        # пропускаем __init__ предка, ибо не поведение переопределено
        BaseFinder.__init__(self, *args, **kwargs)



# -*- coding: utf-8 -*-

import time
import hashlib


DEFAULT_STATSD_HOST = 'localhost'
DEFAULT_STATSD_PORT = 8125


def create_statsd_client(settings):
    if getattr(settings, 'ENABLE_METRICS_COLLECTION', False):
        return StatsdClient(settings)
    else:
        return FakeStatsdClient()


class TimingManager(object):

    def __init__(self, client, controller, request):
        self._client = client
        self._controller = controller
        self._request = request

    def __enter__(self):
        self.time = time.time()
        return self

    def __exit__(self, ex, ex_cls, tb):
        self._client.process(self._controller, self._request, self.time)
        return ex is None


class StatsdClient(object):

    def get_hash(self, s):
        md5 = hashlib.md5()
        md5.update(s)
        str_hash = md5.hexdigest()
        return str_hash

    def __init__(self, settings):
        try:
            import pystatsd
        except ImportError as ie:
            raise ImportError(
                'Metrics collection is enabled, but we failed to '
                'import "pystatsd": {0}'.format(unicode(ie))
            )
        prefix = getattr(settings, 'METRICS_PREFIX', None)
        host = getattr(settings, 'METRICS_HOST', DEFAULT_STATSD_HOST)
        port = getattr(settings, 'METRICS_PORT', DEFAULT_STATSD_PORT)
        self._client = pystatsd.Client(host, port, prefix=prefix)

    def __call__(self, controller, request):
        return TimingManager(self, controller, request)

    def process(self, controller, request, time):
        controller_hash = self.get_hash(controller.url)
        url_hash = self.get_hash(request.path)

        prefix = 'controller_{0}.url_{1}.method_{2}'.format(
            controller_hash, url_hash, request.method.lower())

        self._client.incr('{0}.requests.count'.format(prefix))
        self._client.incr('requests.total')
        self._client.timing_since('{0}.requests.timing'.format(prefix), time)


class FakeStatsdClient(object):

    def __call__(self, controller, request):
        return TimingManager(self, controller, request)

    def process(self, *args, **kwargs):
        pass



#coding:utf-8
'''
Вспомогательные функции используемые в паках
++++++++++++++++++++++++++++++++++++++++++++
'''
import json

from django.db.models.query_utils import Q
from django.db import models

from exceptions import ApplicationLogicException


def apply_sort_order(query, columns, sort_order):
    '''
    Закладывает на запрос порядок сортировки.
    Сначала если в описании колонок columns есть code,
    то сортируем по нему, иначе если есть по name.
    Если задан sort_order, то он главнее всех.

    :param query: запрос, к которому накладывается порядок
        сортировки
    :type query: django.db.models.query.QuerySet

    :param columns: список колонок
    :type columns: dict или tuple

    :param list sort_order: если не пустой, то сортирует по нему

    :return: запрос с наложенной сортировкой
    :rtype: django.db.models.query.QuerySet
    '''
    if isinstance(sort_order, list):
        query = query.order_by(*sort_order)
    else:
        order = None
        for column in columns:
            if isinstance(column, tuple):
                if column[0] == 'name':
                    order = 'name'
                elif column[0] == 'code':
                    order = 'code'
                    break
            elif isinstance(column, dict):
                name = column.get('data_index')
                if name == 'name':
                    order = 'name'
                elif name == 'code':
                    order = 'code'
                    break

        query = query.order_by(order) if order else query.all()
    return query


def create_search_filter(filter_text, fields):
    """
    Фильтрация производится по списку полей fields
    и введеному пользователем тексту filter_text.
    работает по следующей схеме
        Пример:
            fields = ['name', 'family']
            filter_text = u'Вася Пупкин'
        Получится условие WHERE:
            (name like '%Вася%' or family like '%Вася%') and
            (name like '%Пупкин%' or family like '%Пупкин%')
    если один из параметров пуст, то возвращает пустой Q()

    :param str filter_text: текст, пот которому ведется поиск

    :param list fields: список полей, по которым нужно искать

    :return: фильтр, реализующий поиск слов по полям
    :rtype: django.db.models.query_utils.Q
    """
    filter_all = Q()
    if filter_text and fields:
        words = filter_text.split()
        for word in words:
            filter_word = Q()
            for field in fields:
                filter_word |= Q(**{field + '__icontains': word})
            filter_all &= filter_word
    return filter_all


def apply_search_filter(query, filter, fields):
    '''
    Накладывает фильтр поиска на запрос.
    Вхождение каждого элемента фильтра ищется в заданных полях.
    @param query: django.db.models.query.QuerySet
    @param filter: Строка фильтра
    @param fields: Список полей модели по которым будет поиск
    '''
    assert isinstance(fields, (list, tuple))
    condition = create_search_filter(filter, fields)
    if condition is not None:
        query = query.filter(condition)
    return query


def detect_related_fields(query, list_columns):
    """
    Определяет необходимость выполнения select_related на запросе.

    :param query: запрос, к которому применяется select_related
    :type query: django.db.models.query.QuerySet


    """
    # Люди любят указывать в list_columns такие вещи как client.contragent
    # и вообще цеплять вложенные модели. Специально для них генерируем удобный
    # одиночный запрос, который выбирает все что нужно.
    related_models = []
    for column in list_columns:
        if isinstance(column, (tuple, list)):
            column_name = column[0]
        elif isinstance(column, str):
            column_name = column
        elif isinstance(column, dict):
            column_name = column.get('data_index')
            if not column_name:
                continue
        else:
            continue

        if '.' in column_name:
            column_fields = column_name.split('.')
            model_name = column_fields[0]

            # Считаем все что не является последним после точки моделью
            for deep_model in column_fields[1:-1]:
                model_name = model_name + '__' + deep_model

            related_models.append(model_name)

    new_query = query.select_related(*related_models)
    return new_query


def bind_object_from_request_to_form(
        request, obj_factory, form, request_id_name='id', exclusion=None):
    '''
    Функция извлекает объект из запроса по id,
    создает его экземпляр и биндит к форме

    :param request: Запрос от клиента содержащий id объекта
    :type request: django.http.Request

    :param obj_factory: Функция возвращающая объект по его id
    :type obj_factory: callable-object

    :param form: Класс формы к которому привязывается объект
    :type form:

    :param request_id_name: Имя параметра запроса соответствующего ID объекта
    :type request_id_name:

    '''
    exclusion = exclusion or []

    # Получаем объект по id
    id = extract_int(request, request_id_name)
    obj = obj_factory(id)
    if obj is None:
        raise ApplicationLogicException(
            'Такого элемента не существует, возможно он был удален.'
            ' Обновите содержимое.'
        )
    # Разница между новым и созданным объектов в том,
    # что у нового нет id или он пустой
    create_new = True
    if isinstance(obj, dict) and obj.get('id') is not None:
        create_new = False
    elif hasattr(obj, 'id') and getattr(obj, 'id') is not None:
        create_new = False
    # Устанавливаем параметры формы
    win = form(create_new=create_new)
    # Биндим объект к форме
    win.form.from_object(obj, exclusion)
    return win


def bind_request_form_to_object(
        request, obj_factory, form, request_id_name='id', exclusion=None):
    '''
    Функция создает объект по id в запросе
    и заполняет его атрибуты из данных пришедшей формы
    @param request:     Запрос от клиента содержащий id объекта
    @param obj_factory: Функция возвращающая объект по его id
    @param form:        Класс формы к которому привязывается объект
    @param request_id_name:   Имя параметра запроса соответствующего ID объекта
    '''
    exclusion = exclusion or []
    # Получаем наш объект по id и биндим форму к нему
    id = extract_int(request, request_id_name)
    obj = obj_factory(id)

    # Разница между новым и созданным объектов в том,
    # что у нового нет id или он пустой
    create_new = True
    if isinstance(obj, dict) and obj.get('id') is not None:
        create_new = False
    elif hasattr(obj, 'id') and getattr(obj, 'id') is not None:
        create_new = False

    # Создаем форму для биндинга к ней
    win = form(create_new=create_new)
    win.form.bind_to_request(request)
    win.form.to_object(obj, exclusion)

    # Может возникнуть трудноуловимая ошибка, когда в request был id=0 и
    # он присвоился obj.id. В БД не должно быть pk=0
    if not id:
        obj.id = None

    return obj


def safe_delete_record(model, id=None):
    '''
    Безопасное удаление записи в базе.
    В отличие от джанговского ORM не удаляет каскадно.
    Возвращает True в случае успеха, иначе False
    @deprecated нужно использовать BaseModel.safe_delete()
        или m3.db.safe_delete(obj)
    '''
    import warnings
    warnings.warn((
        'ui.actions.utils.safe_delete_record(Model, id)'
        'must by replaced with m3.db.safe_delete(obj)'),
        DeprecationWarning,
        stacklevel=2
    )

    from m3.db import safe_delete
    assert (isinstance(model, models.Model) or issubclass(model, models.Model))
    assert (isinstance(id, int) or isinstance(id, long) or id is None)

    if isinstance(model, models.Model):
        obj = model
    else:
        try:
            obj = model.objects.get(pk=id)
        except model.DoesNotExist:
            #не нашли значит уже удалили
            return True
    return safe_delete(obj)


def fetch_search_tree(
        model_or_query, filter, branch_id=None, parent_field_name='parent'):
    '''
    По заданному фильтру filter и модели model
    формирует развернутое дерево с результатами поиска.
    Если filter пустой, то получается полностью развернутое дерево.
    '''
    # branch_id - это элемент ограничивающий дерево,
    # т.е. должны возвращаться только дочерние ему элементы
    # Сначала тупо получаем все узлы подходящие по фильтру
    assert isinstance(
        model_or_query,
        (models.query.QuerySet, models.Manager)
    ) or issubclass(model_or_query, models.Model)

    if isinstance(model_or_query, (models.query.QuerySet, models.Manager)):
        query = model_or_query
        model = model_or_query.model
    else:
        query = model_or_query.objects
        model = model_or_query
    if branch_id and hasattr(model, 'get_descendants'):
        branch_node = query.get(id=branch_id)
        nodes = branch_node.get_descendants().select_related(parent_field_name)
    else:
        nodes = query.all().select_related(parent_field_name)

    if filter:
        nodes = nodes.filter(filter)

    # Из каждого узла создаем полный путь до корня
    paths = []
    processed_nodes = set()
    for node in nodes:
        # Проверяем не входит ли наш узел в один из просчитанных путей
        if node in processed_nodes:
            continue

        path = [node]
        while getattr(node, parent_field_name):
            if branch_id and getattr(node, parent_field_name) == branch_node:
                break
            if node == getattr(node, parent_field_name):
                break
            node = getattr(node, parent_field_name)
            path.append(node)
            processed_nodes.add(node)
        # Первый элемент пути всегда корень
        path.reverse()
        paths.append(path)

    if len(paths) == 0:
        return []

    def create_one_tree(path):
        ''' Превращает путь в дерево с понимаемое ExtJS гридом '''
        tree = path[0]
        for i in range(1, len(path)):
            path[i - 1].children = [path[i]]
        return tree

    # Начальное дерево в удобоваримом для грида формате
    tree = [create_one_tree(paths[0])]

    # Слияние путей в одно дерево
    def merge(sub_tree, path_slice):
        try:
            index = sub_tree.index(path_slice[0])
        except ValueError:
            sub_tree.append(create_one_tree(path_slice))
        else:
            if hasattr(sub_tree[index], 'children'):
                merge(sub_tree[index].children, path_slice[1:])
            else:
                # Значит сливаемый путь оказался длиннее чем высота поддерева
                sub_tree[index].children = [create_one_tree(path_slice[1:])]

    for path in paths[1:]:
        merge(tree, path)

    def set_tree_attributes(sub_tree):
        '''
        Пробегает дерево и устанавливает узлам атрибуты expanded и leaf
        '''
        if not hasattr(sub_tree, 'children') or len(sub_tree.children) == 0:
            # не факт, что это лист,
            # т.к. мы лишь фильтровали дерево - нужно проверить
            if hasattr(sub_tree, 'is_leaf_node'):
                sub_tree.leaf = sub_tree.is_leaf_node()
            else:
                child_nodes = query.filter(
                    Q(**{parent_field_name: sub_tree.id}))
                if len(child_nodes) == 0:
                    sub_tree.leaf = True
        else:
            sub_tree.expanded = True
            for st in sub_tree.children:
                set_tree_attributes(st)

    for sub_tree in tree:
        set_tree_attributes(sub_tree)

    return tree


def extract_int(request, key):
    """
    Извлекает целое число из запроса

    :param str key: имя параметра

    :return: целочисленное значение параметра с заданным именем
    :rtype: int
    """
    try:
        value = request.REQUEST.get(key, None)
    except IOError:
        # В некоторых браузерах (предполагается что в ie) происходит следующие:
        # request.REQUEST читается и в какой-то момент связь прекращается
        # из-за того, что браузер разрывает соединение,
        # в следствии этого происходит ошибка
        # IOError: request data read error

        #logger.warning(str(err))
        raise

    if value:
        # если по каким-то причинам пришло не число, а что-то другое
        try:
            value = int(value)
        except ValueError:
            raise ApplicationLogicException(
                'Произошла ошибка при конвертации.')
        return value
    else:
        return 0


def extract_int_list(request, key):
    ''' Извлекает список целых чисел из запроса
        ..note:: разделителем в строком значении должна быть запятая

    :param unicode key: имя параметра

    :return: список целочисленных значений из параметра с
        заданным именем
    :rtype: list
    '''
    value = request.REQUEST.get(key, '')
    values = map(int, value.split(','))
    return values


def extract_list(request, key):
    data = request.POST.get(key)
    if data:
        obj = json.loads(data)
        if not isinstance(obj, list):
            obj = [obj]
        return obj
    return []


def apply_column_filter(query, request, map):
    '''
    Накладывает колоночный фильтр

    :param query: Запрос
    :type query: django.db.models.query.QuerySet

    :param request: Данные с клиента включающие фильтры

    :param map: карта связи фильтров в request и полей в объекте:
        ключ - поле объекта, значение - параметр фильтра, например:
            {'unit__name':'unit_ref_name'}
    '''
    assert isinstance(map, dict)
    cond = None
    for key, value in map.items():
        filter_word = request.REQUEST.get(value)
        if filter_word:
            filter = Q(**{key + '__icontains': filter_word})
            cond = filter if not cond else (cond & filter)
    if cond:
        query = query.filter(cond)
    return query



#coding:utf-8
'''
Модуль, реализущий работу с контекстом выполнения операции
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'''
#author: akvarats
import datetime
import json
from decimal import Decimal

from django.utils.encoding import force_unicode
try:
    from django.utils.log import logger
except ImportError:
    from django.utils.log import getLogger
    logger = getLogger('django')


def _date2str(*args):
    # lazy import
    from m3 import date2str
    return date2str(*args)

#============================= ИСКЛЮЧЕНИЯ =====================================
class ActionContextException(Exception):
    """ Базовый класс для исключений контекста """
    pass


class RequiredFailed(ActionContextException):
    u'''
    Исключительная ситуация, которая выбрасывается в случае
    если фактическое наполнение контекста действия не соответствует
    описанным правилам
    '''
    def __init__(self, reason):
        self.reason = reason


class ConversionFailed(ActionContextException):
    u"""
    Исключение, которое выбрасывается,
    если значение из запроса *value*
    не удалось привести к типу *type*,
    указанному в правиле ActionContextDeclaration
    """
    def __init__(self, value, type, *args, **kwargs):
        super(ConversionFailed, self).__init__(*args, **kwargs)
        self.value = value
        self.type = type

    def __str__(self):
        return u'Can not convert value of "%s" in a given type "%s"' % (
            self.value, self.type)


class ContextBuildingError(ActionContextException):
    u"""
    Ошибка построения контекста
    """
    def __init__(self, requiremets=None, errors=None):
        u"""
        :param unicode requiremets: обязательные параметры контекста,
            отсутствующие в запросе
        :param unicode errors: параметры, значение которых
            не удалось распарсить
        """
        assert requiremets or errors, (
            "requiremets or errors must be provided!")
        # параметры должны быть либо в "ошибках", либо в "недостающих"
        assert not set(errors).intersection(requiremets), (
            "requiremets can't contain errors!")
        self.errors = errors or []
        self.requiremets = requiremets or []

    def __repr__(self):
        return "%s(requiremets=%r, errors=%r)" % (
            self.__class__.__name__,
            self.requiremets,
            self.errors
        )

    __str__ = __repr__

    def __unicode__(self):
        log = []
        for title, data in (
            (u"Отсутствуют обязательные параметры:", self.requiremets),
            (u"Неверно заполнены параметры:", self.errors)
        ):
            if data:
                log.append(title)
                log.extend([("- %s" % d) for d in data])
        return u"\n".join(log)


class CriticalContextBuildingError(ContextBuildingError):
    u"""
    Критическая ошибка построения контекста
    """
    pass


#================================== ПАРСЕРЫ ===================================
def _make_datetime_parser(extractor, formats):
    def parse(value):
        for (fmt, length) in formats:
            try:
                result = extractor(
                    datetime.datetime.strptime(value[:length], fmt)
                )
                break
            except ValueError:
                pass
        else:
            raise AssertionError()
        return result
    return parse


def _make_simple_parser(result_type, exceptions=(ValueError, TypeError)):
    def parse(value):
        try:
            result = result_type(value)
        except exceptions:
            raise AssertionError()
        return result
    return parse


def _object_parser(value):
    try:
        result = json.loads(value)
    except ValueError:
        raise AssertionError()
    return result


def _bool_parser(value):
    return value in ('true', 'True', 1, '1', 'on', True)


_datetime_parser = _make_datetime_parser(lambda x: x, formats=(
    ('%Y-%m-%dT%H:%M:%S', 19),
    ('%Y-%m-%d %H:%M:%S', 19),
    ('%d.%m.%Y %H:%M:%S', 19),
    ('%Y-%m-%dT%H:%M', 16),
    ('%Y-%m-%d %H:%M', 16),
    ('%d.%m.%Y %H:%M', 16),
    ('%Y-%m-%d', 10),
    ('%d.%m.%Y', 10),
    ('%H:%M:%S', 8),
    ('%H:%M', 5),
)),


_date_parser = _make_datetime_parser(
    lambda x: x.date(),
    formats=(
        ('%Y-%m-%d', 10),
        ('%d.%m.%Y', 10),
        ('%m/%d/%Y', 10),
    ))


_time_parser = _make_datetime_parser(
    lambda x: x.time(),
    formats=(
        ('%H:%M:%S', 8),
        ('%H:%M', 5),
    ))


_PARSERS = {
    str: _make_simple_parser(unicode),  # Иду на поводу у хомячков (FIXME)
    unicode: _make_simple_parser(unicode),
    int: _make_simple_parser(int),
    float: _make_simple_parser(float),
    Decimal: _make_simple_parser(Decimal),

    object: _object_parser,

    bool: _bool_parser,

    datetime.datetime: _make_datetime_parser(lambda x: x, formats=(
        ('%Y-%m-%dT%H:%M:%S', 19),
        ('%Y-%m-%d %H:%M:%S', 19),
        ('%d.%m.%Y %H:%M:%S', 19),
        ('%Y-%m-%dT%H:%M', 16),
        ('%Y-%m-%d %H:%M', 16),
        ('%d.%m.%Y %H:%M', 16),
        ('%Y-%m-%d', 10),
        ('%d.%m.%Y', 10),
        ('%H:%M:%S', 8),
        ('%H:%M', 5),
    )),

    datetime.date: _make_datetime_parser(lambda x: x.date(), formats=(
        ('%Y-%m-%d', 10),
        ('%d.%m.%Y', 10),
        ('%m/%d/%Y', 10),
    )),

    datetime.time: _make_datetime_parser(lambda x: x.time(), formats=(
        ('%H:%M:%S', 8),
        ('%H:%M', 5),
    )),
}


#================================== КЛАССЫ ====================================
class ActionContextDeclaration(object):
    u"""
    Класс, который определяет правило извлечения параметра из
    запроса и необходимость его наличия в объекте контекста ActionContext.

    :param str name: имя параметра

    :param type: тип извлекаемого значения

    :param bool required: указывает что параметр обязательный

    :param default: значение параметра по умолчанию, используется если его нет
        в запросе, но наличие обязательно

    :param unicode verbose_name: человеческое имя параметра,
        необходимо для сообщений об ошибках
    """
    def __init__(
            self, name='', default=None, type=None,
            required=False, verbose_name='', *args, **kwargs):
        assert type, 'type must be defined!'
        self.name = name
        self.default = default
        self.required = required
        self.type = type
        self.verbose_name = verbose_name

    def human_name(self):
        """
        Возвращает человеческое название параметра *verbose_name*
        """
        return self.verbose_name if self.verbose_name else self.name


class ActionContext(object):
    u'''
    Контекст выполнения операции, восстанавливаемый из запроса.
    '''
    #: Для совместимости
    RequiredFailed = RequiredFailed

    class ValuesList():
        u"""
        Класс для описания параметров, которые будут передаваться в виде
        списка значений, разделенных определенным символом
        """
        def __init__(self, separator=',', type=int, allow_empty=True):
            # разделитель элементов
            self.separator = separator
            # тип, к которому будут преобразовываться эл-ты списка
            self.type = type
            self.allow_empty = allow_empty

    def __init__(self, **kwargs):
        u"""
        Параметры kwargs для быстрой инициализации
        """
        for k, v in kwargs.items():
            setattr(self, k, v)

    def convert_value(self, raw_value, arg_type):
        u'''
        Возвращает значение *raw_value*,
        преобразованное в заданный тип *arg_type*
        '''
        try:
            if isinstance(arg_type, ActionContext.ValuesList):
                elements = raw_value.split(arg_type.separator)
                if not arg_type.allow_empty:
                    elements = [elem for elem in elements if elem]
                value = [
                    self.convert_value(e, arg_type.type)
                    for e in elements
                ]
            else:
                if not arg_type in _PARSERS:
                    raise TypeError('Unknown parser "%r"!' % arg_type)
                value = _PARSERS[arg_type](raw_value)
        except AssertionError:
            raise ConversionFailed(value=raw_value, type=arg_type)

        return value

    def build(self, request, rules):
        u'''
        Выполняет заполнение собственных атрибутов
        согласно переданному запросу, исходя из списка правил

        :param request:запрос, на основе которого производится
            заполнение контекста
        :type reques: django.http.Request

        :param rules: правила извлечения контекста из запроса
        :type rules: список m3_core.actions.context.ActionContextDeclaration
        '''

        params = {}
        if rules:
            for rule in rules:
                # [тип параметра; признак того, что параметр включен в context]
                params[rule.name] = [rule.type, False]

        key = value = ptype = None
        try:
            # переносим параметры в контекст из запроса
            for key in request.REQUEST:
                value = request.REQUEST[key]
                # Пустые параметры не конвертируем,
                # т.к. они могут вызвать ошибку
                if not value:
                    continue

                if key in params:
                    ptype = params[key][0]
                    value = self.convert_value(value, ptype)
                    # Флаг того, что параметр успешно расшифрован
                    # и добавлен в контекст
                    params[key][1] = True
                setattr(self, key, value)
        except IOError as err:
            # В некоторых браузерах (предполагается что в ie)
            # происходит следующие:
            # request.REQUEST читается и в какой-то момент связь прекращается
            # из-за того, что браузер разрывает соединение,
            # в следствии этого происходит ошибка
            # IOError: request data read error
            logger.warning(str(err))
        except ValueError as err:
            # если ошибка преобразования,
            # то пусть проставится значение по-умолчанию
            logger.exception(
                u'Ошибка при преобразовании значения '
                u'из запроса %s="%s" к типу "%s"'
                % (key, value, ptype)
            )

        # переносим обязательные параметры, которые не встретились в запросе
        for rule in rules if rules else []:
            if rule.required and rule.default is not None and (
                    not params[rule.name][1]):
                # если параметр не передан в запросе, но
                # он является обязательным и задано значение по умолчанию,
                # то помещаем этот параметр в контекст
                setattr(self, rule.name, rule.default)
        # проверяем наличие обязательных параметров
        self.check_required(rules)

    def check_required(self, rules):
        u"""
        Проверяет наличие обязательных параметров

        :param rules: правила извлечения контекста из запроса
        :type rules: список m3_core.actions.context.ActionContextDeclaration

        :raise: ActionContext.RequiredFailed
        """
        if not rules:
            return
        for rule in rules:
            if rule.required and getattr(self, rule.name, None) is None:
                raise ActionContext.RequiredFailed(rule.human_name())

    def json(self):
        u"""
        Рендеринг контекста в виде javascript объекта
        """
        def encoder_extender(obj):
            if isinstance(obj, datetime.datetime):
                result = _date2str(obj)
            # WTF? А где время в верхней строке?
            if isinstance(obj, datetime.date):
                result = _date2str(obj)
            elif isinstance(obj, datetime.time):
                result = _date2str(obj, '%H:%M')
            else:
                result = force_unicode(obj)

            return result

        # в json попадут только публичные атрибуты контекста,
        # и при этом только те, которые не callable
        data = dict(
            (k, v)
            for k, v in self.__dict__.iteritems()
            if not (
                k.startswith('_') or
                callable(v)
            )
        )
        return json.dumps(data, default=encoder_extender)

    def combine(self, context):
        u"""
        Объединение контекстов друг с другом.

        :param context: контекст, который объединяеться с текущим
        :type context: m3_core.actions.context.ActionContext

        :return: новый экземпляр контекста, который получился в результате
            слияния с текущим. Все существовавшие значения сохраняются.
        :rtype: m3_core.actions.context.ActionContext
        """
        result = ActionContext()
        if context:
            result.__dict__.update(context.__dict__)
        result.__dict__.update(self.__dict__)
        return result


#-----------------------------------------------------------------------------
class DeclarativeActionContext(ActionContext):
    """
    ActionContext, использующий декларативное описание контекста
    """
    # "встроенные" парсеры
    _parsers = {
        # булево значение
        'boolean': ('true', 'True', '1', 'on', 'yes').__contains__,

        # json в виде строки
        'json': json.loads,

        # дата/время
        'datetime': _datetime_parser,
        'date': _date_parser,
        'time': _time_parser,

        # простые типы
        'int': int,
        'float': float,
        'str': str,
        'unicode': unicode,
        'decimal': Decimal,
    }

    _mode = None

    def build(self, request, rules):
        """
        Выполняет заполнение собственных атрибутов
        согласно переданному запросу, исходя из списка правил

        :param request:запрос, на основе которого производится
            заполнение контекста
        :type reques: django.http.Request

        :param rules: правила извлечения контекста из запроса
        :type rules: список m3_core.actions.context.ActionContextDeclaration

        :raise: TypeError, ContextBuildingError, CriticalContextBuildingError
        """
        assert self.matches(rules), "rules must be a dict or pair!"

        # определяем режим, если правилла описаны парой
        if isinstance(rules, tuple):
            # режим
            mode = request.REQUEST.get(rules[0])
            try:
                # правила для конкретного режима
                rules = rules[1][mode]
            except KeyError:
                raise TypeError('Неизвестный режим: %r=%r' % (rules[0], mode))
            # ну и запоминаем режим
            self._mode = mode

        # аккумуляторы ошибок, связанных с нехваткой и неправильным форматом
        requiremets = []
        errors = []
        only_noncritical = True

        for key, parser_data in rules.iteritems():
            parser = parser_data['type']
            if not callable(parser):
                try:
                    parser = self._parsers[parser]
                except KeyError:
                    raise TypeError(
                        'Неизвестный парсер контекста: "%s"' %
                        parser
                    )

            add_error_to = None
            try:
                val = request.REQUEST.get(key)
                if val is None:
                    if 'default' in parser_data:
                        val = parser_data['default']
                    else:
                        # параметр обязателен, но не присутствует в запросе
                        add_error_to = requiremets
                else:
                    val = parser(val)
            except (ValueError, TypeError, KeyError, IndexError):
                # ошибка преобразования
                add_error_to = errors

            if add_error_to is not None:
                add_error_to.append(
                    parser_data.get('verbose_name', key))
                # ошибка критична, если хотя бы один из параметров
                # не имеет verbose_name
                only_noncritical = only_noncritical and (
                    'verbose_name' in parser_data)
                continue

            setattr(self, key, val)

        if requiremets or errors:
            if only_noncritical:
                raise ContextBuildingError(requiremets, errors)
            else:
                raise CriticalContextBuildingError(requiremets, errors)

    @classmethod
    def register_parser(cls, name, parser):
        """
        Регистрация парсера

        :param parser: парсер
        :type parser: callable-object

        :param unicode name: имя, подо которым регистрируется парсер
        """
        assert callable(parser), "@parser must be a callable object"
        cls._parsers[name] = parser

    @classmethod
    def matches(self, data):
        """
        Возвращает True, если объект data "похож" на правила для
        DeclarativeActionContext
        :param data: проверяемый объект
        :param type: object

        :return: True, если data "похож" на набор правил
        :rtype: boolean
        """
        return isinstance(data, dict) or (
            isinstance(data, tuple) and
            len(data) == 2 and
            isinstance(data[0], basestring)
        )
